#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <sched.h>
#include <time.h>
#include <ctype.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <fcntl.h>

#include <readline/readline.h>
#include <readline/history.h>

#include <sys/msg.h>

#include "phal_hw_api.h"

#include "phal_daemons.h"

#include "cmdman_backend.h"


int _loadapp (char *x);
int _runapp (char *x);
int _pauseapp (char *x);
int _initapp (char *x);
int _stepapp (char *x);
int _stopapp (char *x);
int _statset (char *x);
int _statget (char *x);
int _goout (char *x);
int _help (char *x);
int _hwcmd (char *x);

int done = 0;

typedef struct
{
  char *name;
  rl_icpfunc_t *func;
  char *doc;
} COMMAND;

COMMAND commands[] = {
  {"help", _help, "Display this text"},
  {"hwcmd", _hwcmd, "Hardware-API command. Param: platform command"},
  {"phload", _loadapp, "Load application. Param: app_name"},
  {"phrun", _runapp, "Run application. Param: app_name"},
  {"phinit", _initapp, "Init application. Param: app_name"},
  {"phpause", _pauseapp, "Pause application. Param: app_name"},
  {"phstep", _stepapp, "Step application. Param: app_name"},
  {"phstop", _stopapp, "Stop application. Param: app_name"},
  {"statset", _statset, "Modify statistic variable"},
  {"statget", _statget, "Get a statistic variable value"},
  {"exit", _goout, "Exit this program"},
};

COMMAND *find_command ();

char *stripwhite ();
void print_frame (char *title);



char *
dupstr (char *s)
{
  char *r;

  r = malloc (strlen (s) + 1);
  strcpy (r, s);
  return (r);
}




/*************
 *   USAGE   *
 *************/
void
usage (char *prog)
{
  printf ("%s [-g]\n", prog);

  exit (0);
}







/************
 *   MAIN   *
 ************/
int
main (int argc, char **argv)
{
  char *line, *s;


  initialize_readline ();

  if (!cmdman_init ())
    {
      xprintf ("\nCMDMAN: Error initiating CMDMAN BACKEND\n");
      exit (-1);
    }

  fflush (stdout);
  
  sleep_ms(500);

  printf ("\n\n");

  print_frame ("P-HAL Command Manager");

  printf ("\n\nEnter Commands.\nType 'help' for available commands.\n\n");

  while (!done)
    {
      fflush (stdout);
      
      line = readline ("runph$: ");
      
      if (!line)
	break;

      s = stripwhite (line);

      if (*s)
	{
	  add_history (s);
	  execute_line (s);
	}

      free (line);
    }

  exit (0);

}



int
_goout (char *x)
{
  done = 1;
  return 0;
}

#define MAX_ARGS 10
char *argv[MAX_ARGS];

char app_name[128];
char obj_name[128];
char stat_name[128];
int data[32];



int 
_hwcmd (char *x)
{
	int argc = string_to_argv(x, argv, MAX_ARGS);
	
	cmdman_hwcmd(argv[0]);
}


int
_statset (char *x)
{
  int argc;
  int value;
  
  argc = string_to_argv (x, argv, MAX_ARGS);
  
  if (!get_param (argc, argv, "-a", app_name, PARAM_STRING))
    {
      xprintf ("\nCMDMAN: Error in command, expecting application name (-a)\n");
    }

  if (!get_param (argc, argv, "-o", obj_name, PARAM_STRING))
    {
      xprintf ("\nCMDMAN: Error in command, expecting object name (-o)\n");
    }

  if (!get_param (argc, argv, "-s", stat_name, PARAM_STRING))
    {
      xprintf ("\nCMDMAN: Error in command, expecting stat name (-s)\n");
    }

  /** @TODO: Allow vectors in stats modification 
   */
  if (!get_param (argc, argv, "-d", &value, PARAM_INT))
    {
      xprintf ("\nCMDMAN: Error in command, expecting stat name (-s)\n");
    }

  if (cmdman_statset (app_name, obj_name, stat_name, &value, 1, 15) > 0)
    {
      xprintf ("\nCMDMAN: Stat set ok.\n\n");
    }
  else
    {
      xprintf ("\nCMDMAN: Error setting stat\n\n");
    }

  return 0;
}

int
_statget (char *x)
{
  int argc;
  int type, size;
  
  argc = string_to_argv (x, argv, MAX_ARGS);
  
  if (!get_param (argc, argv, "-a", app_name, PARAM_STRING))
    {
      xprintf ("\nCMDMAN: Error in command, expecting application name (-a)\n");
    }

  if (!get_param (argc, argv, "-o", obj_name, PARAM_STRING))
    {
      xprintf ("\nCMDMAN: Error in command, expecting object name (-o)\n");
    }

  if (!get_param (argc, argv, "-s", stat_name, PARAM_STRING))
    {
      xprintf ("\nCMDMAN: Error in command, expecting stat name (-s)\n");
    }
  
  if (cmdman_statget (app_name, obj_name, stat_name, data, &type, &size, 15) > 0)
    {
      xprintf ("\nCMDMAN: Stat got ok (%d type %d size %d).\n", data[0], type, size);
    }
  else
    {
      xprintf ("\nCMDMAN: Error getting stat\n");
    }
  return 0;
}

int
_loadapp (char *x)
{
  int n;
  
  if (!*x)
    {
      printf ("\nCMDMAN: Error in command. Must enter application name.\n");
      return 0;
    }

  printf ("\nCMDMAN: Loading application \"%s\"...\n\n", x);
  
  n = cmdman_loadapp (x, 15);
  if (n > 0)
    {
      printf ("\nCMDMAN: Application Loaded ok.\n\n");
    }
  else
    {
      printf ("\nCMDMAN: Error while loading application (%d)\n", n);
    }
  return 1;
}

int
_runapp (char *x)
{
  int n;

  if (!*x)
    {
      printf ("\nCMDMAN: Error in command. Must enter application name.\n");
      return 0;
    }

  printf ("\nCMDMAN: Running application \"%s\"...\n\n", x);

  n = cmdman_appstatus (x, PHAL_STATUS_RUN, 15);
  if (n > 0)
    {
      printf ("\nCMDMAN: Application changed ok.\n\n");
    }
  else
    {
      printf ("\nCMDMAN: Error changing application status.\n\n");
    }
  

  return 1;

}

int
_initapp (char *x)
{
  int n;

  if (!*x)
    {
      printf ("\nCMDMAN: Error in command. Must enter application name.\n");
      return 0;
    }

  printf ("\nCMDMAN: Initializing application \"%s\"...\n\n", x);

  n = cmdman_appstatus (x, PHAL_STATUS_INIT, 15);
  if (n > 0)
    {
      printf ("\nCMDMAN: Application changed ok.\n\n");
    }
  else
    {
      printf ("\nCMDMAN: Error changing application status.\n\n");
    }
  

  return 1;

}

int
_pauseapp (char *x)
{
  int n;

  if (!*x)
    {
      printf ("\nCMDMAN: Error in command. Must enter application name.\n\n");
      return 0;
    }

  printf ("\nCMDMAN: Pausing application \"%s\"...\n\n", x);

  n = cmdman_appstatus (x, PHAL_STATUS_PAUSE, 15);
  if (n > 0)
    {
      printf ("\nCMDMAN: Application changed ok.\n\n");
    }
  else
    {
      printf ("\nCMDMAN: Error changing application status.\n\n");
    }
  

  return 1;

}

int
_stepapp (char *x)
{
  int n;

  if (!*x)
    {
      printf ("\nCMDMAN: Error in command. Must enter application name.\n\n");
      return 0;
    }

  printf ("\nCMDMAN: Stepping application \"%s\"...\n\n", x);

  n = cmdman_appstatus (x, PHAL_STATUS_STEP, 15);
  if (n > 0)
    {
      printf ("\nCMDMAN: Application changed ok.\n\n");
    }
  else
    {
      printf ("\nCMDMAN: Error changing application status.\n\n");
    }
  

  return 1;

}

int
_stopapp (char *x)
{
  int n;

  if (!*x)
    {
      printf ("Error in command. Must enter application name.\n");
      return 0;
    }

  printf ("\nCMDMAN: Stopping application \"%s\"...\n\n", x);

  n = cmdman_appstatus (x, PHAL_STATUS_STOP, 15);
  if (n > 0)
    {
      printf ("\nCMDMAN: Application changed ok.\n\n");
    }
  else
    {
      printf ("\nCMDMAN: Error changing application status.\n\n");
    }
  

  return 1;

}

int
_help (char *x)
{
  register int i;
  int printed = 0;

  for (i = 0; commands[i].name; i++)
    {
      if (!*x || (strcmp (x, commands[i].name) == 0))
	{
	  printf ("%s\t\t%s.\n", commands[i].name, commands[i].doc);
	  printed++;
	}
    }

  if (!printed)
    {
      printf ("No commands match `%s'.  Possibilties are:\n", x);

      for (i = 0; commands[i].name; i++)
	{
	  /* Print in six columns. */
	  if (printed == 6)
	    {
	      printed = 0;
	      printf ("\n");
	    }

	  printf ("%s\t", commands[i].name);
	  printed++;
	}

      if (printed)
	printf ("\n");
    }
  return (0);

}










																						      /* Execute a command line. */
int
execute_line (line)
     char *line;
{
  register int i;
  COMMAND *command;
  char *word;

  /* Isolate the command word. */
  i = 0;
  while (line[i] && whitespace (line[i]))
    i++;
  word = line + i;

  while (line[i] && !whitespace (line[i]))
    i++;

  if (line[i])
    line[i++] = '\0';

  command = find_command (word);

  if (!command)
    {
      fprintf (stderr, "%s: No such command.\n", word);
      return (-1);
    }

  /* Get argument to command, if any. */
  while (whitespace (line[i]))
    i++;

  word = line + i;

  /* Call the function. */
  return ((*(command->func)) (word));
}

																															       /* Look up NAME as the name of a command, and return a pointer to that
																															          command.  Return a NULL pointer if NAME isn't a command name. */
COMMAND *
find_command (name)
     char *name;
{
  register int i;

  for (i = 0; commands[i].name; i++)
    if (strcmp (name, commands[i].name) == 0)
      return (&commands[i]);

  return ((COMMAND *) NULL);
}

																																		       /* Strip whitespace from the start and end of STRING.  Return a pointer
																																		          into STRING. */
char *
stripwhite (string)
     char *string;
{
  register char *s, *t;

  for (s = string; whitespace (*s); s++)
    ;

  if (*s == 0)
    return (s);

  t = s + strlen (s) - 1;
  while (t > s && whitespace (*t))
    t--;
  *++t = '\0';

  return s;
}

																																							     /* **************************************************************** */
																																							     /*                                                                  */
																																							     /*                  Interface to Readline Completion                */
																																							     /*                                                                  */
																																							     /* **************************************************************** */

char *command_generator __P ((const char *, int));
char **fileman_completion __P ((const char *, int, int));

																																							     /* Tell the GNU Readline library how to complete.  We want to try to
																																							        complete on command names if this is the first word in the line, or
																																							        on filenames if not. */
initialize_readline ()
{
  /* Allow conditional parsing of the ~/.inputrc file. */
  rl_readline_name = "RunPH";

  /* Tell the completer that we want a crack first. */
  rl_attempted_completion_function = fileman_completion;
}

																																									   /* Attempt to complete on the contents of TEXT.  START and END
																																									      bound the region of rl_line_buffer that contains the word to
																																									      complete.  TEXT is the word to complete.  We can use the entire
																																									      contents of rl_line_buffer in case we want to do some simple
																																									      parsing.  Returnthe array of matches, or NULL if there aren't any. */
char **
fileman_completion (text, start, end)
     const char *text;
     int start, end;
{
  char **matches;

  matches = (char **) NULL;

  /* If this word is at the start of the line, then it is a command
     to complete.  Otherwise it is the name of a file in the current
     directory. */
  if (start == 0)
    matches = rl_completion_matches (text, command_generator);

  return (matches);
}

																																															 /* Generator function for command completion.  STATE lets us
																																															    know whether to start from scratch; without any state
																																															    (i.e. STATE == 0), then we start at the top of the list. */
char *
command_generator (text, state)
     const char *text;
     int state;
{
  static int list_index, len;
  char *name;

  /* If this is a new word to complete, initialize now.  This
     includes saving the length of TEXT for efficiency, and
     initializing the index variable to 0. */
  if (!state)
    {
      list_index = 0;
      len = strlen (text);
    }

  /* Return the next name which partially matches from the
     command list. */
  while (name = commands[list_index].name)
    {
      list_index++;

      if (strncmp (name, text, len) == 0)
	return (dupstr (name));
    }

  /* If no names matched, then return NULL. */
  return ((char *) NULL);
}

																																																											/* **************************************************************** */



void
print_frame (char *title)
{
  printf ("======================================\n");
  printf ("-= %s =-\n", title);
  printf ("======================================\n");

}
