




/** @defgroup res File Resource Management Functions
 * 
 * Accessing on-disk files for reading application, statistics or parameter
 * configurations may be realized through this group of functions.
 *  
 * @{
 */

/** Open a file, parse it to a buffer and close.
 *
 * Useful for reading configuration files. Reads a text
 * file, parses it (erase spaces, remove comments, etc.)
 * and saves it into an allocated buffer of memory. The
 * allocation is done inside this function.
 * The file is also closed by this function when job is 
 * finished and data is saved into memory. 
 *
 * @warnign This function requires the utils library to be 
 * linked to the program.
 *
 * @param res_name Name of the resource (file)
 * @param buffer Address of the pointer where allocated data
 * will be saved. 
 * @return Amount of bytes saved into memory, -1 if error
 */
 int
hwapi_res_parseall (char *res_name, char **buffer);

/** Open Resource (File).
 *
 * @todo path of resource files
 *
 * This function opens a resource so it can be read/written 
 * 
 * The function returns a file descriptor which will be later used
 * by read/write functions.
 *  
 * @param res_name String indicating the name of the resource
 * @param mode FLOW_READ_ONLY, FLOW_READ_WRITE, FLOW_WRITE_ONLY global constants
 * @param file_len fills with length of the file 
 *
 * @return >0 file descriptor
 * @return <0 error
 */
int hwapi_res_open(char *res_name, int mode, int *file_len);

/** Close Resource.
 *	
 * Closes and deallocates resources associated with file descriptor fd.
 * 
 * No further read/write is possible after this call.
 * 
 * @param fd File descriptor
 *
 * @return >0 success
 * @return <0 error
 */
int hwapi_res_close(int fd);


/** Read Resource.
 *
 * Reads blen bytes from the resource fd and saves to user buffer
 * 
 * @param fd file descriptor 
 * @param buffer pointer where data must be saved
 * @param blen number of bytes to read
 *
 * @return >0 number of bytes successfully read
 * @return <0 error
 */
int hwapi_res_read(int fd, char *buffer, int blen);


/** Read Line of Resource (File) Function:
 * 
 * @depreciated
 *
 * Reads a text line (terminated by '\n') from the resource fd. 
 * User must provide buffer length (blen)
 * 
 */
/*int hwapi_read_line_res(int fd, char *buffer, int blen);*/

/** Write Resource.
 *	
 * Writes blen bytes from the user buffer to the resource 
 * associated with file descpriptor
 * 
 * @param fd file descriptor 
 * @param buffer pointer where data must be read
 * @param blen number of bytes to write
 *
 * @return >0 number of bytes successfully read
 * @return <0 error
 */
int hwapi_res_write(int fd, char *buffer, int blen);

/** @} */



