

/* PATH Configurations */

#define TS_ERROR_FRAC	10

/* default path for daemons config file */
#define DEFAULT_DAEMONS_FILE "/usr/local/etc/daemons.conf"

/* default path for xitf config file */
#define DEFAULT_XITF_FILE "/usr/local/etc/xitf.conf"

/* base phal path */
#define DEFAULT_RES_PATH "/etc/phal/"

/* path for lock files */
#define LOCK_PATH "/tmp/"

#define LOCK_FILE "phal.lock"

/* path to downloaded new processes */
#define RCV_PROC_PATH "/tmp/"

#define DEFAULT_CPU_MIPS	0.0

/* Shared memory configuration */

#define HW_API_SHM_KEY	0x1301
#define HW_API_MSG_KEY	0x1281
#define DATA_MSG_KEY	0x1391


/** structure for keeping path of daemons to launch
 */
struct launch_daemons
{
  char path[128];
  char output[128];
  int pid;
};




#define MAX_EXT_ITF 10

/* external network interfaces config db */
struct ext_itf {
	unsigned char id;	/* 0x00 for control */
	int mode;		/* in (1), out (2)*/
	int port;
	char address[15];
	int pid;
	int bpid;
	int fsock;
	int fd;
	int in_mbox;
	int out_mbox;
	int type;
	struct hwapi_xitf_i info;
};



/** Network Database
*/
struct net_db {
	int nof_ext_itf;
	pid_t net_chld;
	struct ext_itf ext_itf[MAX_EXT_ITF];
};








/* database sizes */
#define MAX_PROCS	50
#define MAX_DAEMONS	10
#define MAX_INT_ITF	50
#define MAX_VARS	50

#define VAR_TABLE_SZ		(16*1024)


/** Internal interface database
*/
struct int_itf {
	int id;		/**< Interface Id */
	char r_obj[32];	/**< Reader Object name */
	char w_obj[32];	/**< Writer Object name */
	char r_itf[32];/**< Reader interface name */
	char w_itf[32];/**< Writer interface name */
	int opts;	/**< Interface options */
};




/** Common HW-API database */
struct hw_api_db
{
  /** network db (see below */
  struct net_db net_db;

  /** daemons structure */
  struct launch_daemons daemons[MAX_DAEMONS];

  /** internal interfaces  */
  struct int_itf int_itf[MAX_INT_ITF];

  /** process database */
  struct hwapi_proc_i proc_db[MAX_PROCS];

  /** variable database */
  struct hwapi_var_i var_db[MAX_VARS];

  /* variable data contents */
  char var_table[VAR_TABLE_SZ];

  /** cpu information */
  struct hwapi_cpu_i cpu_info;

  /** path to file resources */
  char res_path[128];

  /** zero time reference */
  time_t init_time;

  int virtual_plt;
  
  
};











#define MSG_MAGIC	0x1321



/** Runph Daemon message commands
 */
#define HWD_SETUP_XITF			0xf000
#define HWD_CREATE_XITF			0xf001
#define HWD_DAEMON_INIT			0xf002
#define HWD_RESTART_XITF 		0xf003
#define HWD_CREATE_PROCESS		0xf004
#define HWD_REMOVE_PROCESS		0xf005
#define HWD_READ_XITF_CFG		0xf006
#define HWD_RUN_PROCESS			0xf007


/* some constants for internal messaging
*/
#define TSTAMP_PKT_BITS		4
#define TSTAMP_PKT_MASK		0xF	/* 0b11*/
#define TSTAMP_MAX_RED		15	/* 2^bits*/


/* packet message header */
struct msghead {
    long dst;   /* message type, must be > 0 */
    long src;
    int opts;
    int tstamp;
};

/** message buffer size */
#define MSG_BUFF_SZ	0x800 /* 8kbytes MSGMAX system limit */

/** message buffer structure */
struct msgbuff {
   struct msghead head;
   int body[MSG_BUFF_SZ];
};

#define MSG_SZ		(sizeof(struct msgbuff))
#define HEAD_SZ		(sizeof(struct msghead))

