#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define DEFAULT_PROTOCOL 0

/********************
 *   SETUP CLIENT   *
 ********************/	
int setup_client(unsigned long int ip, int port)
{
	int n;
	int clientFd, serverLen;
	struct sockaddr_in serverINETAddress;
	struct sockaddr *serverSockAddrPtr;

	serverSockAddrPtr = (struct sockaddr *)&serverINETAddress;
	serverLen = sizeof(serverINETAddress);

	bzero((char *)&serverINETAddress,serverLen);
	serverINETAddress.sin_family = AF_INET;
	serverINETAddress.sin_addr.s_addr = ip;
	serverINETAddress.sin_port = htons(port);
		
	clientFd = socket (AF_INET, SOCK_STREAM, DEFAULT_PROTOCOL);
	if (clientFd == -1)
	{
		perror("socket");
		return(-1);
	}
	
	/*fprintf(out_file,"TESTBED: Connecting...\n");*/
	n=connect(clientFd, serverSockAddrPtr, serverLen);
	if (n)
	{
		perror("connect");
		return(-1);
	}
	/*printf("Connected with %s\n",inet_ntoa(serverINETAddress.sin_addr));*/

	return(clientFd);
}

/********************
 *   SETUP SERVER   *
 ********************/	
int setup_server(unsigned long int ip, int port, int *sck_fd)
{
	int n;
	int serverFd, serverLen, clientLen;
	int socketFd=0;
	struct sockaddr_in serverINETAddress;
	struct sockaddr_in clientINETAddress;
	struct sockaddr *serverSockAddrPtr;
	struct sockaddr *clientSockAddrPtr;

	serverLen = sizeof(serverINETAddress);
	clientLen = sizeof(clientINETAddress);

	bzero((char *)&serverINETAddress,serverLen);
	serverINETAddress.sin_family = AF_INET;
	serverINETAddress.sin_addr.s_addr = ip;
	serverINETAddress.sin_port = htons(port);
	
	serverSockAddrPtr = (struct sockaddr*)&serverINETAddress;
	clientSockAddrPtr = (struct sockaddr *)&clientINETAddress;
	
	/*get a new socket fd if not yet obtained*/
	if (sck_fd!=NULL)
		socketFd=*sck_fd;
		
	if (socketFd<=0)
	{
		socketFd = socket(AF_INET, SOCK_STREAM, DEFAULT_PROTOCOL);
		if (socketFd==-1)
		{
			perror("socket");
			exit(-1);
		}
		n=bind(socketFd, serverSockAddrPtr, serverLen);
		if (n==-1)
		{
			perror("error");
			exit(-1);
		}
		if (sck_fd!=NULL)
			*sck_fd=socketFd;
	}	
		
	/*Get a different file descriptor for each connection*/
	listen (socketFd, 1);
	serverFd = accept(socketFd, clientSockAddrPtr, &clientLen);
	if (serverFd==-1) 
	{
		perror("accept");
		exit(-1);
	}
	/*printf("Connection from %s\n",inet_ntoa(clientINETAddress.sin_addr));*/
	
	return(serverFd);
}
