///////////////////////////
// DECODING part
/////////////////////////

/*Exemple de RS (255,251) */
#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>


#define K_LENGTH 239 // Gràndaria del msg (sense padding) 
#define N_LENGTH 255

#define NPAR (N_LENGTH-K_LENGTH)
#define MAXDEG (NPAR*2)

#define TRUE 1
#define FALSE 0

typedef unsigned long BIT32;
typedef unsigned short BIT16;

/* Decoder syndrome bytes */
int synBytes[MAXDEG];

/* generator polynomial */
int genPoly[MAXDEG*2];
int pBytes[MAXDEG];

/* The Error Locator Polynomial, also known as Lambda or Sigma. Lambda[0] == 1 */
static int Lambda[MAXDEG];

/* The Error Evaluator Polynomial */
static int Omega[MAXDEG];

/* error locations found using Chien's search*/
static int ErrorLocs[256];
static int NErrors;

/* erasure flags */
static int ErasureLocs[256];
static int NErasures;

int gexp[512];
int glog[256];




///////////////////////////////////
//// Galois Field Operations
///////////////////////////////////

/* Inicialitzacio lookup tables */
void init_exp_table (void)
{
	int i, z;
	int pinit,p1,p2,p3,p4,p5,p6,p7,p8;

	pinit = p2 = p3 = p4 = p5 = p6 = p7 = p8 = 0;
	p1 = 1;
	
	gexp[0] = 1;
	gexp[255] = gexp[0];
	glog[0] = 0;			/* shouldn't log[0] be an error? */
	

	//calcul de taula exp, utilitza un LFSR amb el polinomi /* x^8 + x^4 + x^3 + x^2 + 1 */
	for (i = 1; i < 256; i++) {
		pinit = p8;
		p8 = p7;
		p7 = p6;
		p6 = p5;
		p5 = p4 ^ pinit;
		p4 = p3 ^ pinit;
		p3 = p2 ^ pinit;
		p2 = p1;
		p1 = pinit;
		gexp[i] = p1 + p2*2 + p3*4 + p4*8 + p5*16 + p6*32 + p7*64 + p8*128;
		gexp[i+255] = gexp[i];	//pq duplica la taula? 
  	}
	

	//calcul de la taula inversa (log) // COM ?¿?
	for (i = 1; i < 256; i++) {
		for (z = 0; z < 256; z++) {
			if (gexp[z] == i) {
				glog[i] = z;
				break;
			}
		}
	}
}

/* multiplication using logarithms */
int gmult(int a, int b)
{
	int i,j;
	if (a==0 || b == 0) return (0);
	i = glog[a];
	j = glog[b];
	return (gexp[i+j]);
}
/* inversa */
int ginv (int elt) 
{ 
	return (gexp[255-glog[elt]]);
}

/* polynomial multiplication */
void mult_polys (int dst[], int p1[], int p2[])
{
	int i, j;
	int tmp1[MAXDEG*2];
		
	for (i=0; i < (MAXDEG*2); i++) dst[i] = 0;
		
	for (i = 0; i < MAXDEG; i++) {
		for(j=MAXDEG; j<(MAXDEG*2); j++) tmp1[j]=0;
				
		/* scale tmp1 by p1[i] */
		for(j=0; j<MAXDEG; j++) tmp1[j]=gmult(p2[j], p1[i]);
		/* and mult (shift) tmp1 right by i */
		for (j = (MAXDEG*2)-1; j >= i; j--) tmp1[j] = tmp1[j-i];
		for (j = 0; j < i; j++) tmp1[j] = 0;
				
		/* add into partial product */
		for(j=0; j < (MAXDEG*2); j++) dst[j] ^= tmp1[j];
	}
}




/* copia polinomis*/
void copy_poly (int dst[], int src[]) 
{
	int i;
	for (i = 0; i < MAXDEG; i++) dst[i] = src[i];
}

/* posa polinomi a 0 */
void zero_poly (int poly[]) 
{
	int i;
	for (i = 0; i < MAXDEG; i++) poly[i] = 0;
}

void add_polys (int dst[], int src[]) 
{
	int i;
	for (i = 0; i < MAXDEG; i++) dst[i] ^= src[i];
}

/* multiply by z, i.e., shift right by 1 */
static void mul_z_poly (int src[])
{
	int i;
	for (i = MAXDEG-1; i > 0; i--) src[i] = src[i-1];
	src[0] = 0;
}


void scale_poly (int k, int poly[]) 
{	
	int i;
	for (i = 0; i < MAXDEG; i++) poly[i] = gmult(k, poly[i]);
}
/* Calcul del G(X) */
void compute_genpoly (int nbytes, int genpoly[])
{
	int i, tp[256], tp1[256];
	
	/* multiply (x + a^n) for n = 1 to nbytes */

	zero_poly(tp1);
	tp1[0] = 1; //1·X^0 = 1

	for (i = 1; i <= nbytes; i++) {
		zero_poly(tp);
		tp[0] = gexp[i]; //a^n·X^0		/* set up x+a^n */
		tp[1] = 1; //1·X^1
			
		mult_polys(genpoly, tp, tp1);
		copy_poly(tp1, genpoly);
	}
}

/* given Psi (called Lambda in Modified_Berlekamp_Massey) and synBytes,
   compute the combined erasure/error evaluator polynomial as 
   Psi*S mod z^4
  */
void compute_modified_omega ()
{
	int i;
	int product[MAXDEG*2];
		
	mult_polys(product, Lambda, synBytes);	
	zero_poly(Omega);
	for(i = 0; i < NPAR; i++) Omega[i] = product[i];

}


/* gamma = product (1-z*a^Ij) for erasure locs Ij */
void init_gamma (int gamma[])
{
	int e, tmp[MAXDEG];
		
	zero_poly(gamma);
	zero_poly(tmp);
	gamma[0] = 1;
		
	for (e = 0; e < NErasures; e++) {
		copy_poly(tmp, gamma);
		scale_poly(gexp[ErasureLocs[e]], tmp);
		mul_z_poly(tmp);
		add_polys(gamma, tmp);
	}
}



//////////////////////////////////
///// DECODING
/////////////////////////////////

void decode_data(unsigned char data[], int nbytes)
{
	int i, j, sum;
	for (j = 0; j < NPAR;  j++) {
		sum = 0;
		for (i = 0; i < nbytes; i++) {
			sum = data[i] ^ gmult(gexp[j+1], sum);
		}
		synBytes[j]  = sum;
	}
}

int check_syndrome (void)
{
	int i, nz = 0;
	for (i =0 ; i < NPAR; i++) {
		if (synBytes[i] != 0) nz = 1;
	}
	return nz;
}


//////////////////////////////////////
////// MODIFIED BERLEKAMP MASSEY
/***********************************************************************
 * Berlekamp-Peterson and Berlekamp-Massey Algorithms for error-location
 *
 * From Cain, Clark, "Error-Correction Coding For Digital Communications", pp. 205.
 *
 * This finds the coefficients of the error locator polynomial.
 *
 * The roots are then found by looking for the values of a^n
 * where evaluating the polynomial yields zero.
 *
 * Error correction is done using the error-evaluator equation  on pp 207.
 *
 * hqm@ai.mit.edu   Henry Minsky
 */
////////////////////////////////////

void Modified_Berlekamp_Massey (void)
{	
	int n, L, L2, k, d, i;
	int psi[MAXDEG], psi2[MAXDEG], D[MAXDEG];
	int gamma[MAXDEG];
		
	/* initialize Gamma, the erasure locator polynomial */
	init_gamma(gamma);
	
	/* initialize to z */
	copy_poly(D, gamma);
	mul_z_poly(D);
		
	copy_poly(psi, gamma);	
	k = -1; L = NErasures;
	
	for (n = NErasures; n < NPAR; n++) {
		
		d = compute_discrepancy(psi, synBytes, L, n);
			
		if (d != 0) {
				
			/* psi2 = psi - d*D */
			for (i = 0; i < MAXDEG; i++) psi2[i] = psi[i] ^ gmult(d, D[i]);
		
			if (L < (n-k)) {
				L2 = n-k;
				k = n-L;
				/* D = scale_poly(ginv(d), psi); */
				for (i = 0; i < MAXDEG; i++) D[i] = gmult(psi[i], ginv(d));
					L = L2;
			}
		
			/* psi = psi2 */
			for (i = 0; i < MAXDEG; i++) psi[i] = psi2[i];
    		}
		
    		mul_z_poly(D);
  	}
	
	for(i = 0; i < MAXDEG; i++) Lambda[i] = psi[i];
	compute_modified_omega();

}



int compute_discrepancy (int lambda[], int S[], int L, int n)
{
	int i, sum=0;
		
	for (i = 0; i <= L; i++) 
		sum ^= gmult(lambda[i], S[n-i]);
	return (sum);
}


void compute_next_omega (int d, int A[], int dst[], int src[])
{
	int i;
	for ( i = 0; i < MAXDEG;  i++) {
		dst[i] = src[i] ^ gmult(d, A[i]);
	}
}

/* Finds all the roots of an error-locator polynomial with coefficients
 * Lambda[j] by evaluating Lambda at successive values of alpha. 
 * 
 * This can be tested with the decoder's equations case.
 */

void Find_Roots (void)
{
	int sum, r, k;	
	NErrors = 0;

	for (r = 1; r < 256; r++) {
		sum = 0;
		/* evaluate lambda at r */
		for (k = 0; k < NPAR+1; k++) {
			sum ^= gmult(gexp[(k*r)%255], Lambda[k]);
		}
		if (sum == 0) 
		{ 
			ErrorLocs[NErrors] = (255-r); NErrors++; 
			//if (DEBUG) fprintf(stderr, "Root found at r = %d, (255-r) = %d\n", r, (255-r));
		}
	}
}

int correct_errors_erasures (unsigned char codeword[],  int csize, int nerasures, int erasures[])
{
	int r, i, j, err;
	
	/* If you want to take advantage of erasure correction, be sure to
	set NErasures and ErasureLocs[] with the locations of erasures. 
	*/
	NErasures = nerasures;
	for (i = 0; i < NErasures; i++) ErasureLocs[i] = erasures[i];
	
	Modified_Berlekamp_Massey();
	Find_Roots();
	

	if ((NErrors <= NPAR) && NErrors > 0) { 

	/* first check for illegal error locs */
		for (r = 0; r < NErrors; r++) {
			if (ErrorLocs[r] >= csize) {
				//if (DEBUG) fprintf(stderr, "Error loc i=%d outside of codeword length %d\n", i, csize);
				return(0);
			}
		}

		for (r = 0; r < NErrors; r++) {
			int num, denom;
			i = ErrorLocs[r];
			/* evaluate Omega at alpha^(-i) */
	
			num = 0;
			for (j = 0; j < MAXDEG; j++) 
				num ^= gmult(Omega[j], gexp[((255-i)*j)%255]);
	
			/* evaluate Lambda' (derivative) at alpha^(-i) ; all odd powers disappear */
			denom = 0;
			for (j = 1; j < MAXDEG; j += 2) {
				denom ^= gmult(Lambda[j], gexp[((255-i)*(j-1)) % 255]);
			}
	
			err = gmult(num, ginv(denom));
			//if (DEBUG) fprintf(stderr, "Error magnitude %#x at loc %d\n", err, csize-i);
	
			codeword[csize-i-1] ^= err;
    		}
    		return(1);
  	}
  	else {
		//if (DEBUG && NErrors) fprintf(stderr, "Uncorrectable codeword\n");
		return(0);
  	}
}



////////////////////////////////////////
///// OPERACIONS AMB POLINOMIS
////////////////////////////////////////
 

//////////////////////////////////////////
/////* Introduce a byte error at LOC 
/////////////////////////////////////////
void byte_err (int err, int loc, unsigned char *dst)
{
	printf("Adding Error at loc %d, data %#x\n", loc-1, dst[loc-1]);
	dst[loc-1] ^= err;
}

void mostra_array (unsigned char msg[]){

	int i;
	printf("Mostrant Paraula Codi!\n");
	for (i=0; i < N_LENGTH; i++)
		printf("[%d]:%x, ",i,msg[i]);
	printf("\n\n");
}

void transfcode (unsigned char msg[], int len, unsigned char code []){
	
	int i,k;

	for (i=0; i<((N_LENGTH-len)-8); i++)
	{
		code[i]=0;
	}
	
	for (i=((N_LENGTH-len)-8),k=0; i< (N_LENGTH-8); i++,k++)
	{
		code[i]=msg[k];
	}
	for (i=(N_LENGTH-8); i<N_LENGTH; i++)
	{
		code[i]=0;
	}


//prova2
	/*for (i=((N_LENGTH-len)-8),k=0; i< (N_LENGTH-16); i++,k++)
	{
		code[i]=msg[k];
	}

	for (i=(N_LENGTH-16); i<(N_LENGTH-8); i++)
	{
		code[i]=0;
	}
	for (i=(N_LENGTH-8); i<N_LENGTH; i++)
	{
		code[i]=msg[k];
		k++;
	}*/


//prova3 OK!

	/*for (i=(N_LENGTH-len),k=0; i<N_LENGTH; i++,k++)
	{
		code[i]=msg[k];
	}*/
}

////////////////////////////////////////
///// MAIN PROGRAM
////////////////////////////////////////

int main ()
{	
	unsigned char arriveword[32]={0x1,0x3,0x3,0x4,0x5,0x6,0x7,0x8,0x9,0xa,0xb,0xc,0xd,0xe,0xf,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x1,0x91,0x96,0xd8,0xb2,0x51,0xf0,0xb6};


	int i;
	unsigned char codeword [N_LENGTH];

	int erasures[16];
  	int nerasures = 0;
	

	printf("\nInicialitza lookUp Table\n");
	init_exp_table();	//inicialitzam lookup tables GF(2^8)

	printf("\nCalcula Polinomi Generador\n");
	compute_genpoly (NPAR, genPoly); //Calculam el Polinomi generador

	transfcode(arriveword,32, codeword);
	

	for (i=0; i<8; i++){
		erasures[nerasures++] = i;
	}


	decode_data(codeword, N_LENGTH);
	mostra_array(codeword);	
	
	printf("\n\n");
	if (check_syndrome () != 0) {
		correct_errors_erasures (codeword, N_LENGTH, nerasures, erasures);
		printf("Corrected codeword:\n", codeword);
		mostra_array(codeword);
	}
	
	printf("\n\n\nMessage Transmited:\n");

	for ( i=0; i < K_LENGTH; i++)
		printf("[%d]:%x, ",i,codeword[i]);
	
	printf("\n\n");
	return 0;
}
