#include <stdlib.h>
#include <string.h>

#include <math.h>

#include "phal_sw_api.h"

#define PRINT_TTI

#define PKT_LEN 128
int pkt[PKT_LEN];

char log_str[256];

#define DEB

int numero, array[3];
char frase1[128], frase2[128];

int stat_id;

int main(void)
  {
    int status;
    int goexit;
    int tstamp, last_tstamp, last_warn;
    int i, fd_r, fd_w, n;

    if (!InitPHAL())
      {
        sprintf(log_str, "Error initiating phal\n");
        WriteLog(log_str);
        exit(-1);
      }

    goexit = 0;

    while (!goexit)
      {
        status = Status();

        switch (status)
          {
        case PHAL_STATUS_INIT:

          tstamp = GetTstamp();
          last_tstamp = 0;
          last_warn = 0;

          fd_w = CreateItf("sortida", FLOW_WRITE_ONLY);
          if (fd_w < 0)
            {
              fd_w = 0;
            }
          else
            {
              sprintf(log_str, "WRITE init with fd %d\n", fd_w);
              WriteLog(log_str);
            }

          fd_r = CreateItf("entrada", FLOW_READ_ONLY);
          if (fd_r < 0)
            {
              fd_r = 0;
            }
          else
            {
              sprintf(log_str, "READ init with fd %d\n", fd_r);
              WriteLog(log_str);
            }

          /*
          stat_id = InitStat("my_stat", STAT_TYPE_INT, 1);
          if (stat_id < 0)
            {
              sprintf(log_str, "Error initiating stat\n");
              Log(log_str);
              goexit = 1;
              break;
            }
          */
          
          
          if (!InitParamFile())
            {
              sprintf(log_str, "Error initiating parameters\n");
              WriteLog(log_str);
              goexit = 1;
              break;
            }


          if (!GetParam("array", array, STAT_TYPE_INT, 3))
            {
              sprintf(log_str, "Error getting parameter array\n");
              WriteLog(log_str);
              goexit = 1;
              break;
            }
          
          sprintf (log_str, "Array %d-%d-%d init ok\n", array[0], array[1], array[2]);
          WriteLog (log_str);


          break;

        case PHAL_STATUS_RUN:

          last_tstamp = tstamp;
          tstamp = GetTstamp();
          
/*          if (!GetStatsValue (stat_id, &numero, 1))
            {
              sprintf(log_str, "Error getting stats value");
              Log (log_str);
            }
 */         
          if (fd_w)
            {
              for (i = 0; i < PKT_LEN; i++)
                {
                  pkt[i] = tstamp*1000+i;
                }
              n = WriteItf(fd_w, pkt, PKT_LEN);
              if (n<0)
                {
                  sprintf(log_str, "Error writting to flow %d\n", fd_w);
                  WriteLog(log_str);
                  goexit = 1;
                  break;
                }
              if (!n)
                {
                  sprintf(log_str, "NO PACKET SENT\n");
                  WriteLog(log_str);
                }
              else
                {
#ifdef DEB
                  sprintf(log_str, "send %d at tstamp %d\n", pkt[0], tstamp);
                  WriteLog(log_str);
#endif
                }
            }
          if (fd_r)
            {

              n = ReadItf(fd_r, pkt, PKT_LEN);
              if (n<0)
                {
                  sprintf(log_str, "Error reading from flow %d\n", fd_r);
                  WriteLog(log_str);
                  goexit = 1;
                  break;
                }
              if (!n)
                {
                  sprintf(log_str, "NO PACKET RCV\n");
                  WriteLog(log_str);
                }
              else
                {
                  if (last_tstamp != pkt[0]/1000)
                    {
                      sprintf(log_str, "RETRASAT!\t");
                      WriteLog(log_str);
                    }
#ifdef DEB
                  sprintf(log_str, "recv %d at tstamp %d, last %d\n", pkt[0],
                      tstamp, last_tstamp);
                  WriteLog(log_str);
#endif
                }
            }

          if (tstamp - last_tstamp > 1)
            {
              sprintf(log_str, "M'HE PASSAT %d-%d\n", tstamp, last_tstamp);
              WriteLog(log_str);
            }

          if (tstamp - last_warn > 50)
            {
              sprintf(log_str, "I am running at %d\n", tstamp);
              WriteLog(log_str);
              last_warn = tstamp;
            }
          break;
        default:
          goexit = 1;
          }

        if (!goexit)
          {
            Relinquish(1);
          }
      }

    sprintf(log_str, "Bye Bye...\n");
    WriteLog(log_str);

    ClosePHAL();

    exit(0);
  }

