#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "phal_hw_api.h"

#include "phal_sw_api.h"

#include "phal_daemons.h"
#include "stats.h"
#include "log.h"

#define PHAL_STATUS_ABNSTOP 7

#define PMINIT_MAX_TSLOT 	500
#define PMGET_MAX_TSLOT 	500
#define STINIT_MAX_TSLOT 	500
#define LOGINIT_MAX_TSLOT 	500

int obj_id;
int params_initiated;
int init_done, stop_done;
int relinquish_slots;
time_t run_time[3];
int ts_len;

int my_tstamp;

char aux_string[128];

#define CHECK_TSTAMP

//#define PRINT_OBJ_REL_TIME	100 // nof timeslots (period)

#ifdef CHECK_TSTAMP
int last_tstamp = 0;
int nof_passed = 0;
#endif	/*  */

struct my_fd
{
	int fd;
	int mode;
	int id;
};

#define MAX_OBJ_FD 5
/** Keep opened interfaces */
struct my_fd my_fd[MAX_OBJ_FD];

#define MAX_STATS 10

/** Keep opened statistics */
int my_stats[MAX_STATS];

/** My process information
 */
struct hwapi_proc_i my_obj;

int status;

struct hwapi_cpu_i cpu;


/* Buffers for logs */
char logbuffer1[LOG_LINE_SZ], logbuffer2[LOG_LINE_SZ];
char *logbuffer=logbuffer1;
int logoffset = 0;
int log_created = 0;
char timestr[64];


void flush_logs(void);



/** Given an operation frequency, gives the
 * number of words to be written/read, depending
 * on the platform's time slot duration
 */
int GetTempo(int freq)
{
	int nof_words;
	nof_words = (int) freq *((float) ts_len / 1000000);
	return nof_words;
}

/** Get TimeStamp
 * Obtains current Object Time Stamp
 */
int GetTstamp(void)
{
	return my_tstamp;
}



/** Init P-HAL SW-API Library.
 * This function must be called before calling
 * any other P-HAL function.
 */
int InitPHAL()
{
	init_done = 0;
	params_initiated = 0;

	memset(my_fd, 0, sizeof(struct my_fd) * MAX_OBJ_FD);

	/* Init hwapi */
	if (!hwapi_init()) {
		xprintf("SWAPI: Error initiating HWAPI.\n");
		return 0;
	}

	/* get my process information */
	hwapi_proc_myinfo(&my_obj);

	obj_id = my_obj.obj_id;
	my_tstamp = 0;

	/* get cpu info */
	hwapi_hwinfo_cpu(&cpu);
	//  ts_len = cpu.tslen_usec;

	/* update status */
	hwapi_proc_status_get();

#ifdef CHECK_TSTAMP
	last_tstamp = get_tstamp ();
#endif	/*  */
	
	memset(logbuffer1,0,LOG_LINE_SZ);
	memset(logbuffer2,0,LOG_LINE_SZ);

	// xprintf ("SWAPI: Object %s started with Id 0x%x status 0x%x\n", my_obj.obj_name, my_obj.obj_id, status);

	return (obj_id);
}

#ifdef PRINT_OBJ_REL_TIME
int last_printed = 0;
#endif


/** Relinquish Function.
 *
 * Relinquish the CPU ownership of the object.
 * Object goes sleep until the beggining of the
 * next time stamp.
 */
void Relinquish(int nslots)
{
#ifdef PRINT_OBJ_REL_TIME
	time_t t;
#endif

	if (status == PHAL_STATUS_INIT) {
		init_done = 1;
	} else if (status == PHAL_STATUS_STOP || status == PHAL_STATUS_ABNSTOP) {
		xprintf("SWAPI: Error can't call Relinquish in stop, use ClosePHAL and exit\n");
		ClosePHAL();
		exit(-1);
	}

#ifdef PRINT_OBJ_REL_TIME
	hwapi_proc_myinfo(&my_obj);
	get_time(&t);

	if (get_tstamp() - last_printed > PRINT_OBJ_REL_TIME) {
		xprintf("%s \t\t@ %d - %d \t(%d usec cpu)\n", my_obj.obj_name, get_tstamp(), my_tstamp, my_obj.mean_cpu_usec);
		last_printed = get_tstamp();
	}

#endif
	//if (status != PHAL_STATUS_RUN)
		flush_logs();

#ifdef CHECK_TSTAMP
	last_tstamp++;
	if (last_tstamp >= get_tstamp()) {
		hwapi_relinquish(0);
	} 
#else
	hwapi_relinquish(0);
#endif
	my_tstamp++;
}

/** Get Status.
 *
 * Get current object status. This function does not
 * return until object has something to do, that is,
 * statatus is RUN or the first INIT.
 */
int Status(void)
{
	while (1)

	{

		status = hwapi_proc_status_get();

		/* return status */
		if (status == PHAL_STATUS_RUN) {
			return (PHAL_STATUS_RUN);
		} else if (status == PHAL_STATUS_INIT && !init_done) {
			return (PHAL_STATUS_INIT);
		} else if (status == PHAL_STATUS_STOP || status
				== PHAL_STATUS_ABNSTOP) {
			return (PHAL_STATUS_STOP);
		} else if (status == PHAL_STATUS_STEP) {
			hwapi_proc_status_get();
			hwapi_proc_status_ack(obj_id);
			hwapi_proc_status_new(obj_id, PHAL_STATUS_PAUSE, 0);
			hwapi_proc_status_get();
			return PHAL_STATUS_RUN;
		} else {
			hwapi_relinquish(0);
		}
	}
}

/** Creates a flow.
 *
 * Creates a flow given an interface name. Returns
 * a file descriptor (used later to read/write) on it
 */
int CreateItf(char *itf_name, int mode)
{
	int i, fd, itf_id;

	/* search a free fd in local db */
	for (i = 0; i < MAX_OBJ_FD; i++) {
		if (!my_fd[i].fd) {
			break;
		}
	}

	if (i == MAX_OBJ_FD) {
		xprintf("SWAPI: Error no more space for itfs in local db.\n");
		return -1;
	}

	/* get itf id */
	itf_id = hwapi_itf_find(my_obj.obj_name, itf_name, mode);
	if (!itf_id) {
		xprintf("SWAPI: Error itf %s not found.\n", itf_name);
		return -1;
	}

	/* attach to itf */
	fd = hwapi_itf_attach(itf_id, mode);
	if (fd < 0) {
		xprintf("SWAPI: Error attaching to itf.\n");
		return -1;
	}

	/* save info */
	my_fd[i].id = itf_id;
	my_fd[i].fd = fd;
	my_fd[i].mode = mode;

	return fd;
}

/** Close a flow.
 * @param fd Itf file descriptor
 */
int CloseItf(int fd)
{
	int i;

	/* search itf */
	for (i = 0; i < MAX_OBJ_FD; i++) {
		if (my_fd[i].fd == fd) {
			break;
		}
	}

	if (i == MAX_OBJ_FD) {
		xprintf("SWAPI: Error can't find fd %d\n", fd);
		return 0;
	}

	hwapi_itf_unattach(my_fd[i].fd, my_fd[i].mode);

	memset(&my_fd[i], 0, sizeof(struct my_fd));

	return 1;
}

/** Write a packet in an itf.
 *
 * Puts data pointed by buffer into the
 * interface associated with file descriptor
 *
 * @param fd File Descriptor
 * @param buffer Pointer to data to write
 * @param size Number of words (32-bit) to write
 *
 * @return Number of words written, -1 if error
 */
int WriteItf(int fd, void *buffer, int size)
{
	return hwapi_itf_snd(fd, buffer, size);
}

/** Read a packet from an itf.
 *
 * Receives a packet from the interface.
 * @todo continuous interfaces
 *
 * @param fd File Descriptor
 * @param buffer Pointer where data must be saved.
 * @param size Size in words (32-bit) of the buffer
 *
 * @return Number of words received, -1 if error
 */
int ReadItf(int fd, void *buffer, int size)
{
	return hwapi_itf_rcv(fd, buffer, size);
}

int GetItfStatus(int fd)
{
	return hwapi_itf_status (fd);
}

/** Initiates Parameter Configuration File.
 *
 * Tells STATS daemon to init configuration file for
 * parameters.
 * @return 1 if ok 0 if error
 */
int InitParamFile(void)
{
	int stat_id, opt;
	int count = 0;

	if (params_initiated) {
		xprintf("SWAPI: Error params have been already initiated.\n");
		return 0;
	}

	sprintf(aux_string, "%s:%s", my_obj.app_name, my_obj.obj_name);

	stat_id = hwapi_var_create(aux_string, 1, VAR_IS_PMINIT);
	if (stat_id < 0) {
		xprintf("SWAPI: Error initiating params.\n");
		return 0;
	}

	/* wait to be initiated */
	do {
		opt = hwapi_var_getopt(stat_id);

		if (opt != VAR_IS_PMINITOK && opt != VAR_IS_PMINITERR) {
			hwapi_relinquish(0);
			count ++;
		}
	} while ((opt != VAR_IS_PMINITOK && opt != VAR_IS_PMINITERR) && count
			< PMINIT_MAX_TSLOT);

	if (opt != VAR_IS_PMINITOK) {
		xprintf("SWAPI: Error initiating parameters file.\n");
		hwapi_var_delete(stat_id);
		return 0;
	}

	/* remove variable */
	hwapi_var_delete(stat_id);

	//xprintf ("SWAPI: Parameters initiated ok.\n");

	params_initiated = 1;

	return 1;

}

/** Close Parameter Configuration File.
 *
 * Tells STATS daemon to close configuration file for
 * parameters.
 * @return 1 if ok 0 if error
 */
int CloseParamFile(void)
{
	int stat_id;

	if (!params_initiated) {
		xprintf("SWAPI: Error must initiate params file first.\n");
		return 0;
	}

	sprintf(aux_string, "%s:%s", my_obj.app_name, my_obj.obj_name);

	stat_id = hwapi_var_create(aux_string, 1, VAR_IS_PMCLOSE);
	if (stat_id < 0) {
		xprintf("SWAPI: Error closing params.\n");
		return 0;
	}

	// xprintf ("SWAPI: Parameters closed ok.\n");

	params_initiated = 0;

	return 1;

}

int sizeoftype(int type)
{
	switch (type) {
	case STAT_TYPE_CHAR:
	case STAT_TYPE_UCHAR:
		return sizeof(char);
	case STAT_TYPE_FLOAT:
		return sizeof(float);
	case STAT_TYPE_INT:
		return sizeof(int);
	case STAT_TYPE_SHORT:
		return sizeof(short);
	default:
		xprintf("SWAPI: Error invalid stat type %d\n", type);
		return 0;
	}
}

/** Gets a parameter value.
 *
 * Gets an initialization parameter value.
 *
 * @param ParamName Name of the parameter
 * @param ParamValue Pointer where to save data
 * @param ParamLen Length of the expected parameter.
 *
 * @return 1 if ok, 0 if error
 */
int GetParam(char *ParamName, void *ParamValue, char ParamType, int ParamLen)
{
	int stat_id, opt;
	int count = 0;

	if (!params_initiated) {
		xprintf("SWAPI: Error must initiate params file first.\n");
		return 0;
	}

	stat_id = hwapi_var_create(ParamName, ParamLen * sizeoftype(ParamType),
			VAR_IS_PMGET);
	if (stat_id < 0) {
		xprintf("SWAPI: Error initiating params.\n");
		return 0;
	}

	/* put expected type */
	hwapi_var_write(stat_id, &ParamType, 1);

	/* wait to be initiated */
	do {
		opt = hwapi_var_getopt(stat_id);

		if (opt != VAR_IS_PMVALOK && opt != VAR_IS_PMVALERR) {
			hwapi_relinquish(0);
			count ++;
		}
	} while ((opt != VAR_IS_PMVALOK && opt != VAR_IS_PMVALERR) && count
			< PMGET_MAX_TSLOT);

	if (opt != VAR_IS_PMVALOK) {
		xprintf("SWAPI: Error initiating parameters file.\n");
		hwapi_var_delete(stat_id);
		return 0;
	}

	/* copy data */
	if (hwapi_var_read(stat_id, ParamValue, ParamLen
			* sizeoftype(ParamType)) < 0) {
		xprintf("SWAPI: Error reading variable\n");
		hwapi_var_delete(stat_id);
		return 0;
	}

	/* and delete variable */
	hwapi_var_delete(stat_id);

	//  xprintf ("SWAPI: Parameter %s got ok.\n", ParamName);

	return 1;
}

/** Initializes an statistics variable.
 *
 * Initializes a variable given a name, type and size.
 * Once the variable has been initialized, it can be
 * accessed (to view or modify) by higher layers (CMDMAN, user, etc.).
 *
 * @todo define variable types
 *
 * @param StatName Name of the variable
 * @param StatType Variable Type
 * @param StatSize Size in elements (depends on type) of the variable
 *
 * @return > 0 Stat_id, -1 if error
 */
int InitStat(char *StatName, char StatType, int StatSize)
{

	int stat_id, opt;
	int count = 0;
	int i;

	/* check if available local stats */
	i = 0;
	while (i < MAX_STATS && my_stats[i]) {
		i++;
	}
	if (i == MAX_STATS) {
		xprintf("SWAPI: No more space at local db for stats\n");
		return -1;
	}

	if (StatSize <= 0) {
		xprintf("SWAPI: Error invalid size %d\n", StatSize);
	}

	sprintf(aux_string, "%s:%s:%s", my_obj.app_name, my_obj.obj_name,
			StatName);

	stat_id = hwapi_var_create(aux_string, StatSize * sizeoftype(StatType),
			VAR_IS_STINIT);
	if (stat_id < 0) {
		xprintf("SWAPI: Error initiating stat.\n");
		return -1;
	}

	/* put stat type in variable contents first byte */
	hwapi_var_write(stat_id, &StatType, 1);

	/* wait to be initiated */
	do {
		opt = hwapi_var_getopt(stat_id);

		if (opt != VAR_IS_STOK && opt != VAR_IS_STERR) {
			hwapi_relinquish_daemon();
			count ++;
		}
	} while ((opt != VAR_IS_STOK && opt != VAR_IS_STERR) && count
			< STINIT_MAX_TSLOT);

	if (opt != VAR_IS_STOK) {
		xprintf("SWAPI: Error initiating stat.\n");
		hwapi_var_delete(stat_id);
		return -1;
	}

	my_stats[i] = stat_id;

	//  xprintf ("SWAPI: Stat %s initiated correctly.\n", StatName);

	return stat_id;
}

/** Close an statistic.
 *
 * Closes an stat variable so it can not be anymore used
 * neither by the object or other entities.
 * Resources are also de-allocated.
 *
 * @return 1 ok 0 error
 */
int CloseStat(int StatId)
{
	int i;
	
	if (!StatId) {
		xprintf("SWAPI: Invalid statId %d\n", StatId);
		return 0;
	}

	/* check it has been created */
	i = 0;
	while (i < MAX_STATS && my_stats[i] != StatId) {
		i++;
	}
	if (i == MAX_STATS) {
		xprintf("SWAPI: Unknown stat 0x%x\n", StatId);
		return 0;
	}

	if (!hwapi_var_setopt(StatId, VAR_IS_STCLOSE)) {
		xprintf("SWAPI: Error closing stat.\n");
		return 0;
	}

	my_stats[i] = 0;

	/* stat will deleted by STATS when closing request is ack */

	//  xprintf ("SWAPI: Stat %d closed ok.\n", StatId);

	return 1;

}









/** Creates a log
 *
 * Creates a log for the caller's object
 * *
 * @return > 0 Log_id, -1 if error
 */
int CreateLog()
{

	int stat_id, opt;
	int count = 0;
	int i;
	
	if (log_created) {
		xprintf("SWAPI: Caution: only one log per object is allowed\n");
		return log_created;
	}
	
	sprintf(aux_string, "%s:%s", my_obj.app_name, my_obj.obj_name);
	stat_id = hwapi_var_create(aux_string, LOG_LINE_SZ,
			VAR_IS_LOGINIT);
	if (stat_id < 0) {
		xprintf("SWAPI: Error creating log: Error creating stat.\n");
		return -1;
	}

	/* wait to be initiated */
	do {
		opt = hwapi_var_getopt(stat_id);
		if (opt != VAR_IS_LOGINITOK && opt != VAR_IS_LOGINITERR) {
			hwapi_relinquish_daemon();
			count ++;
		}
	} while ((opt != VAR_IS_LOGINITOK && opt != VAR_IS_LOGINITERR) && count
			< LOGINIT_MAX_TSLOT);
	if (opt != VAR_IS_LOGINITOK) {
		xprintf("SWAPI: Error creating log: Invalid response.\n");
		hwapi_var_delete(stat_id);
		return -1;
	}
	
	//xprintf ("SWAPI: Log with id %d created\n", stat_id);
	log_created = stat_id;
	
	return stat_id;
}

/** Close an statistic.
 *
 * Closes an stat variable so it can not be anymore used
 * neither by the object or other entities.
 * Resources are also de-allocated.
 */
int CloseLog(int LogId)
{
	int i;
	
	if (!LogId) {
		xprintf("SWAPI: Invalid LogId %d\n", LogId);
		return 0;
	}

	if (!hwapi_var_setopt(LogId, VAR_IS_LOGCLOSE)) {
		xprintf("SWAPI: Error closing log.\n");
		return 0;
	}
	
	log_created = 0;

	xprintf ("SWAPI: Log %d closed\n", LogId);
	return 1;
}


void flush_logs(void) {	
	if (logoffset > 0) {
		logbuffer[logoffset] = '\0';
		logoffset++;
		if (hwapi_var_write(log_created, logbuffer, logoffset) < 0) {
			xprintf ("SWAPI: Error writting to log: Invalid id (%d)\n", log_created);
			return;
		}
		
		//printf("flushing length %d\n", strlen(logbuffer)+1);
		hwapi_var_setopt(log_created, VAR_IS_LOGWRITE);
		logbuffer = (logbuffer==logbuffer1)?logbuffer2:logbuffer1;
		logoffset = 0;		
	}
}
	

/** LogWrite
 * Writes a character string to log with id log_id
 * 
 * @return 1 ok 0 error
 */
int WriteLog(int LogId, char *str)
{
	assert(str);
	time_t t;

	struct hwapi_cpu_i cpu;
	hwapi_hwinfo_cpu(&cpu);
	
	if (cpu.disable_logs) {
		return 1;
	}

	if (!*str) {
		return 0;
	}
	
	/* check line is shorter than maximum allowed line */
	if (strlen(str) > LOG_LINE_SZ - 50) {
		xprintf("SWAPI: Error logging, maximum line length exceed (%d>%d)\n", strlen(str), LOG_LINE_SZ);
		return 0;
	}
	
	get_time(&t);
	sprintf(timestr, "\n[%03d:%03d:%03d]:\t", t.tv_sec, t.tv_usec/1000, t.tv_usec%1000);
	//if (LogId==1)
	//	printf("logging %s at [%03d:%03d:%03d]\n", my_obj.obj_name, t.tv_sec, t.tv_usec/1000, t.tv_usec%1000);
	
	/* if space not avaiable in current buffer
	 * send and save in the other */
	if (strlen(str) + strlen(timestr) + logoffset >= LOG_LINE_SZ) {
		if (hwapi_var_getopt(LogId) == VAR_IS_LOGWRITE) {
			xprintf("SWAPI: [%s]: CAUTION! Missed log due to full buffer at %d\n", my_obj.obj_name, get_tstamp());
		}
		flush_logs();

	}
	//strcat(&logbuffer[logoffset], timestr);
	memcpy(&logbuffer[logoffset], timestr, strlen(timestr));
	logoffset += strlen(timestr);

	//strcat(&logbuffer[logoffset], str);
	memcpy(&logbuffer[logoffset], str, strlen(str));
	
	/* erase last \n, if any */
	/*char *c = strstr(&logbuffer[logoffset], "\n");
	if ((c-&logbuffer[logoffset]) == strlen(str) - 1) {
		*c = '\0';
	}
	*/
	logoffset += strlen(str);
	
	//printf("%d write %d off %d\n", logbuffer == logbuffer2, strlen(str) + strlen(timestr), logoffset);
	
	return 1;
}

/** Set stat value.
 *
 * Modifies a variable value.
 *
 * @param StatId Id of the variable
 * @param Value Pointer to user data
 * @param Size In elements, depending on type.
 *
 * @todo read/write size depends on type
 */
int SetStatsValue(int StatId, void * Value, int Size)
{
	struct hwapi_cpu_i cpu;
	hwapi_hwinfo_cpu(&cpu);
	
	if (cpu.disable_stats) {
		return 1;
	}
	return hwapi_var_write(StatId, Value, Size);
}

/** Get stat value.
 *
 * Obtains a variable value. Value is saved in
 * User buffer.
 *
 * @param StatId Id of the variable
 * @param Buffer Pointer to buffer where data must be saved.
 * @param BuffByteSize Size (in bytes) of the user-buffer
 */
int GetStatsValue(int StatId, void * Buffer, int BuffByteSize)
{
	struct hwapi_cpu_i cpu;
	hwapi_hwinfo_cpu(&cpu);
	
	if (cpu.disable_stats) {
		return 1;
	}
	return hwapi_var_read(StatId, Buffer, BuffByteSize);
}

/** Close P-HAL.
 *
 * Closes P-HAL base resources allocated on
 * InitPHAL, but DOES NOT deallocate stats/parameters
 * or interfaces. This should be done manually.
 */
void ClosePHAL()
{
	int i, n;

	/* close all opened stats not closed */
	for (i = 0; i < MAX_STATS; i++) {
		if (my_stats[i]) {
			CloseStat(my_stats[i]);
		}
	}

	/* close parameters if not closed */
	if (params_initiated) {
		CloseParamFile();
	}

	/* Close attached Flows */
	for (i = 0; i < MAX_OBJ_FD; i++) {
		if (my_fd[i].fd) {
			CloseItf(my_fd[i].fd);
		}
	}

	/* And delete own hwapi interfaces */
	hwapi_itf_delete_own(my_obj.obj_name);

	n = hwapi_proc_status_get();
	if (n != PHAL_STATUS_STOP && n != PHAL_STATUS_ABNSTOP) {
		hwapi_proc_status_new(obj_id, PHAL_STATUS_ABNSTOP, -1);
		hwapi_proc_status_get();
		hwapi_proc_status_get();
	}

	/* wait to exit until EXEC daemon confirms our new status (stop) */
	while (!hwapi_proc_status_new(obj_id, PHAL_STATUS_STOP, 0))
		;

	//printf("%d exiting\n", getpid());
	hwapi_exit();
}
