
/** @defgroup cmds_statsman STATS MANAGER Input Commands
 *
 * STATS MANAGER input commands description.
 * @ingroup commands
 * @{
 */

/** @defgroup cmds_statsman_init Stat/Parameters Initialization Request
 *
 * STATS Sensor issues a request to register a statistic variable value (so it can 
 * be viewed/modified by the user through CMDMAN or to initialize the parameters
 * configuration file (where values are hold, it must be done prior to requesting the value) .
 *
 * @{
 */

/** STAT INIT Command Constant
 */
#define STATSMAN_STINIT		1

/** PARAMETERS INIT Command Constant
 */
#define STATSMAN_PMINIT		2

/** LOG INIT Command Constant
 */
#define STATSMAN_LOGINIT	3

int statsman_incmd_init(daemon_o daemon);

/** @} */


/** @defgroup cmds_statsman_close Stat/Parameters Close Request
 *
 * STATS Sensor issues a command requesting to unregister an statistic variable (it can not be
 * used anymore and its resources are freed) or to close a parameter's configuration file (so any
 * more values will be able to be obtained) 
 * @{
 */

/* STAT CLOSE Command Constant
*/
#define STATSMAN_PMCLOSE	4

/** PARAMETERS CLOSE Command Constant 
 */
#define STATSMAN_STCLOSE	5

/** LOG CLOSE Command Constant 
 */
#define STATSMAN_LOGCLOSE	6

int statsman_incmd_close(daemon_o daemon);

/** @} */


/** @defgroup cmds_statsman_stact Stat Value Modify/View User Request 
 *
 * User, through the CMDMAN daemon, requests to view or modiffy a variable
 * previously registered by the Object. 
 * @{
 */ 

/** STAT GET Command Request 
 */
#define STATSMAN_STGET		7
/** STAT SET Command Request 
 */
#define STATSMAN_STSET		8

int statsman_incmd_stact(daemon_o daemon);

/** @} */


/** @defgroup cmds_statsman_stactack Stat Action Acknowledgment
 *
 * STATS Sensor reports the result of an Stat Action Request telling if the 
 * modification or the getting of the variable was successfull or not. In case
 * that the action was GET and the result is success, the value of the variable
 * comes with the command
 *
 * @{
 */
 

/** SET OK Command Constant 
 */
#define STATSMAN_STSETOK	9
/** SET ERROR Command Constant 
 */
#define STATSMAN_STSETERR	10
/** GET OK Command Constant 
 */
#define STATSMAN_STVALOK	11
/** GET ERROR Command Constant 
 */
#define STATSMAN_STVALERR	12

int statsman_incmd_stactack(daemon_o daemon);

/** @} */


/** @defgroup cmds_statsman_pmget Parameter Value Request
 *
 * STATS Sensor reports that an object is requesting a parameter value. 
 *
 * @{
 */

/** PARAMETER REQUEST Command Constant
 */
#define STATSMAN_PMGET		13

int statsman_incmd_pmget(daemon_o daemon);

/**@} */


/** @defgroup cmds_statsman_logwrite Write to log
 *
 * STATS Sensor reports contents to write to the log 
 *
 * @{
 */

/** LOG WRITE Command Constant
 */
#define STATSMAN_LOGWRITE	14

int statsman_incmd_logwrite(daemon_o daemon);
