/** @defgroup cmds_swman SW MANAGER Input Commands
 *
 * SW MANAGER Input commands description.
 * @ingroup commands
 * @{
 */

/** @defgroup cmds_swman_loadapp Load an Application
 *
 * This command is received by CMDMAN Daemon after User orders to load an application.
 * 
 * The application description (list of objects, associated executables, interfaces and their interconnection)
 * is described in the application configuration file (see swman_parser.c).
 *
 * This command, thus, will parse this file and build the list of objects that will be send downwards
 * in the hierarchy.
 *
 * @{
 */

/** LOAD APP Command Constant 
 */
#define SWMAN_LOADAPP		1

int swman_incmd_loadapp(daemon_o daemon);


/** @} */

/** @defgroup cmds_swman_appstatus Change Application Status
 *
 * Change the Execution Status (INIT, RUN, PAUSE, STOP) of a previously loaded application.
 * 
 * @{
 */
 
/** APP STATUS Command Constant
 */
#define SWMAN_APPSTATUS		2

int swman_incmd_appstatus(daemon_o daemon);

/** @} */


/** @defgroup cmds_swman_execreq Executable Binary Request 
 *
 * SWLOAD Requests to download the executable binary contents.
 *
 * @{
 */

/** EXEC REQUEST Command Constant
 */
#define SWMAN_EXECREQ		3

int swman_incmd_execreq(daemon_o daemon);

/** @} */


/** @defgroup cmds_swman_statusrep Object Status Report 
 *
 * EXEC Sensor reports of a successfully change in an object execution status within certain limits (see exec.h). In case
 * the object did not had time to change its status, or it crashed during that phase, STATUSERR command will be reported
 *
 * @{
 */

/** STATUS CHANGE OK Command Constant
 */
#define SWMAN_STATUSOK		4

/** STATUS ERROR Command Constant 
 */
#define SWMAN_STATUSERR		5

int swman_incmd_statusrep(daemon_o daemon);


/** @} */

/** @} */

#define SWMAN_EXECERR		6
int swman_incmd_execerr(daemon_o daemon);
