#ifndef ID_INCLUDED
#define ID_INCLUDED

/** @defgroup id P-HAL Id
 * @ingroup common_phal
 *
 * Id's manipulation is a bit tricky in P-HAL. As using dynamic memory for allocating just
 * a char or even a int would be a bit inefficient, static memory allocation is used for manipulating
 * this types. Depending on the type, id's range from chars (1byte) to integers (4-bytes), unsigned most
 * of the cases. Typedefs define an specific type for each of them, however, and in order to be a coherent
 * with the rest of the "objects", type size should not be need to be known by the programmer, who can access
 * the id_cpy/id_cmp/id_get/id_set functions to perform the required r/w operations.
 *
 * Note that any arithmetic operation has been defined for this types has this is not the intention of 
 * these types, they should be only used to identificate objects, applications, stats, etc.
 *
 * @{
 */ 

typedef unsigned int 	uint;	
#define varid_t		uint
#define itfid_t		uint


typedef unsigned short 	ushort;
#define objid_t		ushort
#define appid_t 	ushort
#define xitfid_t	ushort
#define peid_t		ushort
#define pltid_t		ushort
#define varsz_t		ushort
#define vartype_t       ushort
#define cmd_t		ushort
#define dmid_t		ushort

typedef unsigned char 	uchar;
#define status_t	uchar


/** Assign src contents to dest contents*/
//#define id_cpy(dst,src)		(*dst=*src)

/** Compare contents*/
//#define id_cmp(x,y)		(*x!=*y)

/** Set value*/
//#define id_set(x,val)		(*x=val)

/** Get value*/
//#define id_get(x)		(*x)

/* unprotected functions  */
/*int objid_xcmp (const void *x, const void *y);
int appid_xcmp (const void *x, const void *y);
int pltid_xcmp (const void *x, const void *y);
int varid_xcmp (const void *x, const void *y);
int statusid_xcmp (const void *x, const void *y);
*/
/** @} */ 
  
#undef T
#endif /*  */
