/** @defgroup pkt P-HAL Command Packet
* @ingroup common_phal
*
* This object is used to manage in/out packets.
*
* Indeed, this function manages just buffers, it does
* not neither create them or send/receive them. Just
* a buffer is assigned to them during their creation
* and functions are provided to access both its contents
* and its header (src processor, destination processor
* and daemon, etc).
*
* The contents of the packet are accessed in terms of 
* fields. A field is a portion of data associated 
* with a constant. In the transmission of the packet
* fields are created and associated. In the reception, 
* user can quickly access the contents by just referencing
* the desired field code.
* 
* @{
*/  
  
#ifndef INPKT_INCLUDED
#define INPKT_INCLUDED

#define T pkt_o

#define uint unsigned int

typedef unsigned char fieldcode_t;
typedef unsigned short fieldsz_t;

typedef struct T *T;

typedef int     (*pkt_putfnc) (const void *x, void *data, int max_size);
typedef void *  (*pkt_getfnc) (void *data, int *len);

T 		pkt_new(int *ptr, int buff_sz);
void 		pkt_delete (T * pkt);

int 		pkt_readvalues (T pkt, int size_pkt);
void	 	pkt_clear (T pkt);

int 		pkt_freesz (T pkt);
int		pkt_len (T pkt);
int		pkt_wlen (T pkt);
  
void *		pkt_get (T pkt, fieldcode_t field_code, 
                    void * newfrompkt (char **start, char *end));

void *          pkt_getvalandsz (T pkt, fieldcode_t field_code, int *sz);
void * 		pkt_getval (T pkt, fieldcode_t field_code);
fieldsz_t 	pkt_getvalsz (T pkt, fieldcode_t field_code);
uint		pkt_getvalue (T pkt, fieldcode_t field_code);


int		pkt_put (T pkt, fieldcode_t field_code,
                    void *member, 
                    int topktf (void *x, char **start, char *end));


void * 		pkt_putval (T pkt, fieldcode_t field_code,
		   fieldsz_t val_sz);

int		pkt_putvalue (T pkt, fieldcode_t field_code, 
                    uint value);


void 		pkt_setdestpe (T pkt, peid_t pe);
peid_t  	pkt_getsrcpe (T pkt);

void 		pkt_setdestdaemon (T pkt, dmid_t daemon);

cmd_t 	 	pkt_getcmd (T pkt);
void		pkt_setcmd (T pkt, cmd_t cmd);

/** Daemon packets 
*/ 
  struct header
{
  peid_t src;
  peid_t dst_pe;
  dmid_t dst_daemon;
  cmd_t cmd;
};

#define HEAD_SZ (sizeof(struct header)>>2)

/** @} */ 


#undef T
  
#endif /*  */
