/** @file bridge.c
 *
 * @brief BRIDGE SW Daemon Implementation
 *
 * @todo Comment this daemon.
 */

/* standard libraries */
#include <stdlib.h>
#include <string.h>

#include "phal_hw_api.h"

#include "phal_daemons.h"
#include "phid.h"

#include "pkt.h"
#include "daemon.h"

#include "bridge.h"

/* include command declarations */
#include "bridge_cmds.h"

/* main function declaration */
void bridge_run();

#ifdef RUN_AS_PROC
int main()
{
	bridge_run();
}
#endif

/** BRIDGE Daemon Main Routine
 */
void bridge_run()
{
	daemon_o daemon;

	int n;

	daemon = daemon_new(DAEMON_BRIDGE);
	if (!daemon) {
		xprintf ("swman: Error creating daemon\n");
		exit(-1);
	}

	/*
	n = 0;
	while(!n)
		sleep(1);
	*/
	/** Configure input commands
	 */

	if (daemon_addcmd(daemon, BRIDGE_IDENT, bridge_incmd_ident) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, BRIDGE_ADDITF, bridge_incmd_additf) < 0) {
		exit(-1);
	}
	/*
	if (daemon_addcmd(daemon, BRIDGE_RMITF, bridge_incmd_rmitf) < 0) {
		exit(-1);
	}
	*/
	n=bridge_init();
	if (n<0) {
		xprintf("BRIDGE: Error initiating external itf\n");
		exit(-1);
	}

	xprintf ("BRIDGE: Init with PID=%d PPID=%d\n", getpid(), getppid());

	/**  main daemon loop 
	 * while packets in queue, process them
	 */
	while (1) {

		do {
			n = daemon_rcvpkt(daemon);
			if (n > 0) {
				daemon_processcmd(daemon);
			}
		} while (n);
		if (n < 0) {
			break;
		}

		/** After all commands are processed, we will bridgeute the 
		 * real-time and bridgeution status monitoring
		 */
		bridge_background(daemon);

		hwapi_relinquish_daemon();
	}

	daemon_delete(&daemon);

	exit(-1);
}
