#include <stdlib.h>
#include <string.h>

#include "phal_hw_api.h"

#include "phid.h"
#include "phal_daemons.h"

/* common objects */
#include "xitf.h"
#include "pkt.h"
#include "daemon.h"

#include "bridge.h"

#include "hwman_cmds.h"
#include "bridge_cmds.h"
#include "frontend.h"

#define DEB

#define DEB_PKT

#define DEFAULT_EXT_OUT		1 /* itf number, in case of non-local objects */

#define MAX_IN_ITF	5
#define MAX_OUT_ITF	5

#define MAX_XITF	(MAX_IN_ITF+MAX_OUT_ITF)

#define BRIDGE_MAX_ITF 	10


#define DEF_PKT_DATA_LEN	2048

/*Buffer for any packet*/
int gbuff[DEF_PKT_DATA_LEN];

struct bridge_itf bridge_itf_table[BRIDGE_MAX_ITF];

int nof_in_itf, nof_out_itf;

struct ext_itf ext_in_itf[MAX_IN_ITF];
struct ext_itf ext_out_itf[MAX_OUT_ITF];

static int pe_id = 0;

static struct hwapi_xitf_i xitf[MAX_XITF];

int new_neighbour(daemon_o daemon, int loc_itf_id, int *packet)
{
	struct bridge_ident *bident;
	pkt_o pkt;
	xitf_o xitf;
	
	pkt = daemon_pkt (daemon);

	bident=(struct bridge_ident*) packet;

#ifdef DEB
	xprintf("BRIDGE: New neighbour with id 0x%x (litf=0x%x,oitf=0x%x,C=%.2f)\n",
			bident->pe_id, loc_itf_id, bident->xitf.id,bident->xitf.BW);
#endif

	xitf = xitf_new ();
	if (!xitf) {
		return 0;
	}
	memcpy(&xitf->xitf, &bident->xitf, sizeof (struct hwapi_xitf_i));
	xitf->remote_id = loc_itf_id;
	xitf->remote_pe = bident->pe_id;
	
	/* send to SWMAP */
	pkt_clear (pkt);
	pkt_putvalue (pkt, FIELD_PEID, pe_id);
	pkt_put (pkt, FIELD_XITF, xitf, xitf_topkt);
	xitf_delete(&xitf);
	
	daemon_sendto (daemon, HWMAN_ADDITF, 0, DAEMON_HWMAN);
	
	return 1;
}

int bridge_incmd_ident(daemon_o daemon)
{
	int i, n;
	struct bridge_ident *bident;

	bident=(struct bridge_ident*) gbuff;
	bident->magic=BRIDGE_IDENT_MAGIC;
	bident->pe_id=pe_id;
	
	xprintf ("BRIDGE: Received ident request to %d interfaces\n", nof_out_itf);

	for (i=0; i<nof_out_itf; i++) {
		if (ext_out_itf[i].fd) {
			memcpy((char*) &bident->xitf,
					(char*) &xitf[ext_out_itf[i].xitf_pos],
					sizeof(struct hwapi_xitf_i));

			n = hwapi_itf_snd(ext_out_itf[i].fd, gbuff,
					BIDENT_PKT_SZ);
			if (n<0) {
				xprintf("BRIDGE: Error writting to itf 0x%x\n",
						ext_in_itf[i].id);
				return -1;
			}
			xprintf("BRIDGE: Ident packet sended to 0x%x BW=%.2f!!\n", ext_out_itf[i].id, xitf[ext_out_itf[i].xitf_pos].BW);
		}
	}

	return 1;
}

int bridge_init(void)
{
	int i, n;
	struct hwapi_cpu_i cpu_i;

	memset(bridge_itf_table, 0, sizeof(struct bridge_itf) * BRIDGE_MAX_ITF);
	memset(ext_in_itf, 0, MAX_IN_ITF * sizeof(struct ext_itf));
	memset(ext_out_itf, 0, MAX_IN_ITF * sizeof(struct ext_itf));

	nof_in_itf=0;
	nof_out_itf=0;

	/* wait to be registered */
	do {
		hwapi_hwinfo_cpu(&cpu_i);
		pe_id = cpu_i.pe_id;
		if (!pe_id) {
			hwapi_relinquish_daemon();
		}
	} while (!pe_id);

	n = hwapi_hwinfo_xitf(xitf, MAX_XITF);

	for (i = 0; i < n; i++) {
		if (xitf[i].id & EXT_ITF_DATA_MASK) {
			if (xitf[i].mode == FLOW_READ_ONLY) {
				if (!utils_create_attach_ext(xitf[i].id, 0xf000, &ext_in_itf[nof_in_itf].fd, FLOW_READ_ONLY)) {
					xprintf ("BRIDGE: Error creating itf 0x%x\n", xitf[i].id);
				} else {
					ext_in_itf[nof_in_itf].id = xitf[i].id;		
					ext_in_itf[nof_in_itf].xitf_pos = i; 
					xprintf ("BRIDGE: External input itf 0x%x configured fd %d.\n", xitf[i].id, ext_in_itf[nof_in_itf].fd);
					nof_in_itf++;	
				}
			} else if (xitf[i].mode == FLOW_WRITE_ONLY) {
				if (!utils_create_attach_ext(xitf[i].id, 0xf000, &ext_out_itf[nof_out_itf].fd, FLOW_WRITE_ONLY)) {
					xprintf ("BRIDGE: Error creating itf 0x%x\n", xitf[i].id);
				} else {
					ext_out_itf[nof_out_itf].id = xitf[i].id;
					ext_out_itf[nof_out_itf].xitf_pos = i;
					xprintf ("BRIDGE: External output itf 0x%x configured fd %d.\n", xitf[i].id, ext_out_itf[nof_out_itf].fd);
					nof_out_itf++;	
										
				}
			}
		}
	}
	return 1;
}

int packet_in(int *pbuff, int pktsize, int keep_header)
{
	int n;
	int itf;
	int i;

	/* search destination itf */
	i = 0;
	while (memcmp(pbuff, &bridge_itf_table[i].dest, PH_HEADER_SZ<<2) 
			&& i < BRIDGE_MAX_ITF)
		i++;
	if (i < BRIDGE_MAX_ITF) {
		itf = bridge_itf_table[i].fd_w;
	} else {
		data_header_t *head = (data_header_t*) pbuff;
		/* if not present in local routing table bridge to default external output interface */
		xprintf ("BRIDGE: Error can't find destination 0x%x:0x%x\n", head->ObjId, head->ItfId);
		return 0;
	}

	/* if destination itf is internal don't transmit header */
	if (!keep_header) {
		pbuff += PH_HEADER_SZ;
		pktsize -= PH_HEADER_SZ;
	}

	/* route packet */
	do {
		n = hwapi_itf_snd(itf, pbuff, pktsize);
		if (!n) {
			hwapi_relinquish_daemon();
		}
	} while (!n);

	return (n);
}

int packet_out(int *pbuff, int pktsize, data_header_t *hd, int fd_w,
	int make_header)
{
	int n;

	if (make_header) {
		/* copy header to packet buffer */
		memcpy(pbuff, hd, PH_HEADER_SZ<<2);
		pktsize+=PH_HEADER_SZ;
	} else
		pbuff+=PH_HEADER_SZ;

	/* route packet */
	do {
		n = hwapi_itf_snd(fd_w, pbuff, pktsize);
		if (!n)
			hwapi_relinquish_daemon();
	} while (!n);

	return (n);
}

int bridge_incmd_additf(daemon_o daemon)
{
	int i, j, n;
	pkt_o pkt;
	data_header_t *head;
	int obj_itf_id, ext_itf_id, obj_mode;

	assert(daemon);
	pkt = daemon_pkt(daemon);
	
	head = (data_header_t*) pkt_getval (pkt, FIELD_DATAHEADER);
	if (!head) {
		xprintf ("BRIDGE: Error in add_itf command packet\n");
		return 0;
	}
	obj_itf_id = pkt_getvalue (pkt, FIELD_OBJITFID);
	obj_mode = pkt_getvalue (pkt, FIELD_OBJITFMODE);
	ext_itf_id = pkt_getvalue (pkt, FIELD_OBJXITFID);
	
	if (!obj_itf_id || !obj_mode || !ext_itf_id) {
		xprintf ("BRIDGE: Error in new itf command packet\n");
		return 0;
	}
	/* search if entry already in db */
	i = 0;
	j = BRIDGE_MAX_ITF;
	while ( i < BRIDGE_MAX_ITF && memcmp(&bridge_itf_table[i].dest,
			head, sizeof(data_header_t))) {
		i++;
		if (!bridge_itf_table[i].dest.ObjId) {
			j = i;
			memcpy(&bridge_itf_table[i].dest, head, sizeof(data_header_t));
		}
	}
	if (i == BRIDGE_MAX_ITF) {
		if (j == BRIDGE_MAX_ITF) {
			xprintf ("BRIDGE: Error no more space in interface table\n");
		} else {
			i = j;
		}
	}

	/* attach bridge side of object interface if not yet attached*/
	if (obj_mode == FLOW_WRITE_ONLY) {
		if (bridge_itf_table[i].fd_r) {
			hwapi_itf_unattach(bridge_itf_table[i].fd_r, FLOW_READ_ONLY);
		}
		n = hwapi_itf_attach(obj_itf_id, FLOW_READ_ONLY);
		if (n<0) {
			xprintf("BRIDGE: Error while attaching to itf 0x%x.\n",obj_itf_id);
			memset(&bridge_itf_table[i], 0, sizeof(struct bridge_itf));
			return 0;
		}
		bridge_itf_table[i].fd_r = n;
		xprintf ("attaching %d to 0x%x\n", bridge_itf_table[i].fd_r, obj_itf_id);
		
		j=0;
		while (j < nof_out_itf && ext_out_itf[j].id != ext_itf_id)
			j++;
		if (j == nof_out_itf) {
			xprintf("BRIDGE: Error can't find external itf 0x%x\n",
					ext_itf_id);
			memset(&bridge_itf_table[i], 0, sizeof(struct bridge_itf));
			return 0;
		}
		if (!ext_out_itf[j].fd) {
			xprintf("BRIDGE: External interface must be configurated first (0x%x at %d)\n",
					ext_itf_id, j);
			memset(&bridge_itf_table[i], 0, sizeof(struct bridge_itf));
			return 0;
		}
		bridge_itf_table[i].fd_w = ext_out_itf[j].fd;
	} else {
		if (bridge_itf_table[i].fd_w) {
			hwapi_itf_unattach(bridge_itf_table[i].fd_w, FLOW_WRITE_ONLY);
		}
		n=hwapi_itf_attach(obj_itf_id, FLOW_WRITE_ONLY);
		if (n<0) {
			xprintf("BRIDGE: Error while attaching to itf 0x%x.\n", obj_itf_id);
			memset(&bridge_itf_table[i], 0, sizeof(struct bridge_itf));
			return 0;
		}
		bridge_itf_table[i].fd_w = n;
		
		
	}

	bridge_itf_table[i].mode = obj_mode;
	xprintf("BRIDGE: Table updated. Destination=0x%x:0x%x id=0x%x\n", head->ObjId, head->ItfId, ext_itf_id);

}

int bridge_background(daemon_o daemon)
{
	int n, i;

	for (i=0; i<nof_in_itf; i++) {
		if (ext_in_itf[i].fd) {
			do {
				n=hwapi_itf_rcv(ext_in_itf[i].fd, gbuff,
						DEF_PKT_DATA_LEN);
				if (n<0) {
					xprintf("BRIDGE: Error receiving packet from itf %d.\n",
							i);
					return 0;
				}
				if (!n) {
					break;
				}
				
				if (gbuff[0]==BRIDGE_IDENT_MAGIC) {
					new_neighbour(daemon, ext_in_itf[i].id, gbuff);
					break;
				}
#ifdef DEB_PKT
				xprintf("BRIDGE: Bridged input packet %d words at TS=%d\n",n,get_tstamp());
#endif
				n=packet_in(gbuff, n, 0);
			} while (n>0);
		}
	}

	for (i=0; i<BRIDGE_MAX_ITF; i++) {
		if (bridge_itf_table[i].mode==FLOW_WRITE_ONLY
				&& bridge_itf_table[i].fd_r
				&& bridge_itf_table[i].fd_w) {
			/* receive from object */
			do {
				n=hwapi_itf_rcv(bridge_itf_table[i].fd_r,
						&gbuff[PH_HEADER_SZ],
						DEF_PKT_DATA_LEN-PH_HEADER_SZ);
				if (n<0) {
					xprintf(
							"BRIDGE: Error bridging output data packet (0x%x:0x%x %d mode %d) \n",
							bridge_itf_table[i].dest.ObjId,
							bridge_itf_table[i].dest.ItfId,
							i,
							bridge_itf_table[i].mode);
					return 0;
				}

				if (!n) {
					break;
				}
#ifdef DEB_PKT
				xprintf("BRIDGE: Bridged output packet %d words at TS=%d\n",n,get_tstamp());
#endif
				/* ... and bridge */
				n = packet_out(gbuff, n,
						&bridge_itf_table[i].dest,
						bridge_itf_table[i].fd_w, 1);
			} while (n>0);
		}
	}
}

