#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "phid.h"
#include "phal_daemons.h"

#include "phal_hw_api.h"

#include "str.h"
#include "set.h"

#include "app.h"
#include "phitf.h"
#include "rtcobj.h"
#include "phobj.h"

#define T app_o

int appid = 1;

/** Constructor
 *
 * @return pointer to the object, or NULL if error
 */
T app_new ()
{
	T app;

	NEW (app);
	if (!app) {
		return NULL;
	}

	app->app_id = (appid_t) appid++;
	app->name = (str_o) str_new ();
	app->objects = Set_new (0, NULL);

	if (!app->objects || !app->name)
	{
		app_delete ((T*) &app);
		return NULL;
	}

	return app;
}

/** Destructor
 */
void app_delete(T *app)
{
	assert (app);
	assert (*app);

	if ((*app)->name) {
		str_delete(&(*app)->name);
	}
	if ((*app)->objects) {
		Set_destroy(&(*app)->objects, phobj_xdelete);
	}

	free(*app);
	app = NULL;
}

/** Compare string x with application name in member 
 */
int app_findname(const void *x, const void *member)
{
	T app = (T) member;

	return str_scmp(app->name, (char *) x);
}

/** Compare id x with application id in member
 */
int app_findid(const void *x, const void *member)
{
	T app = (T) member;

	return (*((appid_t *) x) != app->app_id);
}

/** topkt Function
 */
int app_topkt(void *x, char **start, char *end)
{
	T app = (T) x;

	return Set_topkt(app->objects, start, end, phobj_topkt);
}

/** newfrompkt function
 */
void * app_newfrompkt(char **start, char *end)
{
	T app;

	assert (start && end);
	assert (end >= *start);

	app = app_new();
	if (!app) {
		return NULL;
	}

	app->objects = Set_newfrompkt(start, end, phobj_newfrompkt);

	return app;
}

void app_xdelete(void **x)
{
	app_delete((T*) x);
}

void app_sdelete(void **x)
{
	assert (x);
	assert (*x);
	free(*x);
	x = NULL;
}
