#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "phid.h"
#include "phal_daemons.h"

#include "phal_hw_api.h"

#include "str.h"
#include "set.h"
#include "xitf.h"
#include "pe.h"

#define T pe_o


/** Constructor
 *
 * @return pointer to the object, or NULL if error
 */
T pe_new ()
{
	T pe;

	NEW (pe);
	if (!pe) {
		return NULL;
	}

	pe->xitfs= Set_new (0, NULL);
	if (!pe->xitfs)
	{
		pe_delete ((T*) &pe);
		return NULL;
	}

	return pe;
}

/** Destructor
 */
void pe_delete(T *pe)
{
	assert (pe);
	assert (*pe);

	if ((*pe)->xitfs) {
		Set_destroy(&(*pe)->xitfs, xitf_xdelete);
	}

	free(*pe);
	pe = NULL;
}

/** Compare id x with pelication id in member
 */
int pe_findid(const void *x, const void *member)
{
	T pe = (T) member;

	return (*((peid_t *) x) != pe->id);
}

int pe_findPidx(const void *x, const void *member) 
{
	T pe = (T) member;

	return (*((int *) x) != pe->P_idx);
}

struct pkt
{
	peid_t id;
	int plat_family;
	int P_idx;
	int BW_idx;
	float C;
	char is_hwman;
};

#define PE_MIN_PKT_SZ	(1 + sizeof (struct pkt))

/** topkt Function
 */
int pe_topkt(void *x, char **start, char *end)
{
	T pe = (T) x;
	
	assert (pe && start && end);
	assert (*start <= end);

	if ((end - *start) < PE_MIN_PKT_SZ) {
		return 0;
	}

	memcpy(*start, (void *) pe, sizeof(struct pkt));
	*start += sizeof(struct pkt);

	if (!Set_topkt(pe->xitfs, start, end, xitf_topkt)) {
		return 0;
	}
}

/** newfrompkt function
 */
void * pe_newfrompkt(char **start, char *end)
{
	T pe;

	assert (start && end);
	assert (end >= *start);

	if (end - *start < PE_MIN_PKT_SZ) {
		return NULL;
	}

	NEW (pe);
	if (!pe) {
		return NULL;
	}

	memcpy((void *) pe, *start, sizeof(struct pkt));
	*start += sizeof(struct pkt);

	pe->xitfs= (Set_o) Set_newfrompkt(start, end, xitf_newfrompkt);
	if (!pe->xitfs) {
		pe_delete((T*) &pe);
		return NULL;
	}
	
	return pe;
}
