#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "phid.h"
#include "phal_daemons.h"

#include "phal_hw_api.h"

#include "str.h"
#include "set.h"

#include "xitf.h"

#define T xitf_o

/** Constructor
 *
 * @return pointer to the object, or NULL if error
 */
T xitf_new ()
{
	T xitf;

	NEW (xitf);
	if (!xitf) {
		return NULL;
	}

	return xitf;
}

/** Destructor
 */
void xitf_delete(T *xitf)
{
	assert (xitf);
	assert (*xitf);

	free(*xitf);
	xitf = NULL;
}

/** Compare id x with xitf id in member
 */
int xitf_findid(const void *x, const void *member)
{
	T xitf = (T) member;

	return (*((xitfid_t *) x) != xitf->xitf.id);
}

int xitf_findremotepe(const void *x, const void *member)
{
	T xitf = (T) member;

	return (*((xitfid_t *) x) != xitf->remote_pe);
}

#define XITF_MIN_PKT_SZ	(sizeof (struct xitf_o))

/** topkt Function
 */
int xitf_topkt(void *x, char **start, char *end)
{
	T xitf = (T) x;
	
	assert (xitf && start && end);
	assert (*start <= end);

	if ((end - *start) < XITF_MIN_PKT_SZ) {
		return 0;
	}

	memcpy(*start, (void *) xitf, sizeof (struct xitf_o));
	*start += sizeof (struct xitf_o);

}

/** newfrompkt function
 */
void * xitf_newfrompkt(char **start, char *end)
{
	T xitf;

	assert (start && end);
	assert (end >= *start);

	if (end - *start < XITF_MIN_PKT_SZ) {
		return NULL;
	}

	NEW (xitf);
	if (!xitf) {
		return NULL;
	}

	memcpy((void *) xitf, *start, sizeof (struct xitf_o));
	*start += sizeof (struct xitf_o);
	
	return xitf;
}

void xitf_xdelete(void **x)
{
	xitf_delete ((T*) x);
}
