// function that calculates the communication cost for DAGs with any ordering
// FULL-DUPLEX, HALF-DUPLEX, or BUS architectures

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "crm.h"

extern float q2;		// cost function parameter
extern int mhop;		// multi-hop indicator; mhop = 0: single hops only, mhop = 1: 2-hop only if necessary, mhop = 2: single & 2-hops 

extern int N;

void update(float bx, float (*B_rem)[Nmax], int indx1, int indx2);

float cost_comm(float bx1, float bx2, float (*B_rem)[Nmax], int indx1, int indx2)
{
	// local variables
	int x, p, n;			// path indices
	int  sel_path = -1;
	int path_num = 0;		// number of paths
	int num_hops[Nmax];		// number of hops per path
	int path[Nmax][Nmax];			// characterizes the N possible 2-paths, e.g. path[2] = (1 2 3), indicates that path 3 is [P1 P2 P3], a 2-hop path from P1 to P3.
	//unsigned B_eff[Nmax];		// effective bandwidth = minimum bandwith along the path			

	float cost[Nmax];			// path cost
	float cost_total = 0;		// communication cost due to the selected path

	// intialization
	for (x=0; x<N; x++)
		num_hops[x] = 0;

	/* ################################ single hops only ################################ */
	if (mhop == 0)
	{
		if (bx1 > 0)	// non-zero bandwidth requirement
		{
			//if (bx1 > B_rem[indx1][indx2])					// bandwidth requirement greater than bandwidth capacity
			if ((bx1 - B_rem[indx1][indx2])	> threshold)		// bandwidth requirement greater than bandwidth capacity
				cost_total = infinite;							// infinite cost
			else
			{
				cost_total = q2*bx1/B_rem[indx1][indx2];		// cost = required_bandwidth / available_bandwidth
				update(bx1, B_rem, indx1, indx2);			// update remaining bandwidth
			}
		}
		else if (bx2 > 0)										// non-zero bandwidth requirement
		{
			//if (bx2 > B_rem[indx2][indx1])					// bandwidth requirement greater than bandwidth capacity
			if ((bx2 - B_rem[indx2][indx1]) > threshold)		// bandwidth requirement greater than bandwidth capacity
				cost_total = infinite;							// infinite cost
			else
			{
				cost_total = q2*bx2/B_rem[indx2][indx1];		// cost = required_bandwidth / available_bandwidth
				update(bx2, B_rem, indx2, indx1);			// update remaining bandwidth
			}
		}
	}

	/* ########## single and 2-hops (mhop = 1: 2-hops if and only if necessary) ########### */
	else
	{
		if (bx1 > 0)	// non-zero bandwidth requirement for receiving data by module pre-mapped to P[indx2] from module pre-mapped to P[indx1]
		{
			cost_total = infinite;	// there will be a communication cost since bx1 > 0; initiate with infinite
			// ################## 1st-hop ##################
			for (x=0; x<N; x++)
			{
				//if ((x == indx1) || (bx1 > B_rem[indx1][x]))
				if ((x == indx1) || ((bx1 - B_rem[indx1][x]) > threshold))
					;
				else
				{
					//B_eff[path_num] = B_rem[indx1][x];
					path[path_num][num_hops[path_num]] = indx1;
					num_hops[path_num]++;
					path[path_num][num_hops[path_num]] = x;

					cost[path_num] = q2*bx1/B_rem[indx1][x];	// cost function specific
					path_num++;
				}
			}

			if (path_num > 0)
			{
				// ################## 2nd- and FINAL hop ##################
				for (p=0; p<path_num; p++)
				{
					if (path[p][num_hops[p]] == indx2)	// already terminated path (single-hop path)
						;
					else
					{
						//if (bx1 > B_rem[path[p][num_hops[p]]][indx2])
						if ((bx1 - B_rem[path[p][num_hops[p]]][indx2]) > threshold)
							cost[p] = infinite;
						else
						{
							//if (((B_rem[path[p][num_hops[p]]][indx2]) - B_eff[p]) < 0)
				            //    B_eff[p] = B_rem[path[p][num_hops[p]]][indx2];

							if (mhop == 1)
								cost[p] += q2*bx1/B_rem[path[p][num_hops[p]]][indx2] + (float)1000; // cost function specific; + 1000 to potentiate single hops (direct links) and pre-mappings that avoid interprocessor data flow
							else
								cost[p] += q2*bx1/B_rem[path[p][num_hops[p]]][indx2]; // cost function specific

							num_hops[p]++;
						    path[p][num_hops[p]] = indx2;

						}
					}
				}

				// CHOOSE PATH ...
				for (p=0; p<path_num; p++)
				{
					if ((cost_total - cost[p]) > threshold) //(cost[p] < cost_total))
					{
						cost_total = cost[p];
						sel_path = p;
					}
				}

				// UPDATE BANDWIDTH RESOURCES ...
				if (cost_total < (infinite - 1))
				{
					for (n=0; n<num_hops[sel_path]; n++)
						update(bx1, B_rem, path[sel_path][n], path[sel_path][n+1]);
				}

			} // if (path_num > 0)

		} // if (bx1 > 0)

		else if (bx2 > 0)	// non-zero bandwidth requirement for sending data from module pre-mapped to P[indx2] to module pre-mapped to P[indx1]
		{
			cost_total = infinite;	// there will be a communication cost since bx1 > 0; initiate with infinite
			// ################## 1st-hop ##################
			for (x=0; x<N; x++)
			{
				//if ((x == indx2) || (bx2 > B_rem[indx2][x]))
				if ((x == indx2) || ((bx2 - B_rem[indx2][x]) > threshold))
					;
				else
				{
					//B_eff[path_num] = B_rem[indx2][x];
					path[path_num][num_hops[path_num]] = indx2;
					num_hops[path_num]++;
					path[path_num][num_hops[path_num]] = x;

					cost[path_num] = q2*bx2/B_rem[indx2][x];	// cost function specific
					path_num++;
				}
			}

			if (path_num > 0)
			{
				// ################## 2nd- and FINAL hop ##################
				for (p=0; p<path_num; p++)
				{
					if (path[p][num_hops[p]] == indx1)	// already terminated path (single-hop path)
						;
					else
					{
						//if (bx2 > B_rem[path[p][num_hops[p]]][indx1])
						if ((bx2 - B_rem[path[p][num_hops[p]]][indx1]) > threshold)
							cost[p] = infinite;
						else
						{
							//if (((B_rem[path[p][num_hops[p]]][indx1]) - B_eff[p]) < 0)
						    //    B_eff[p] = B_rem[path[p][num_hops[p]]][indx1];

							if (mhop == 1)
								cost[p] += q2*bx2/B_rem[path[p][num_hops[p]]][indx1] + (float)1000; // cost function specific; + 1000 to potentiate single hops (direct links) and pre-mappings that avoid interprocessor data flow
							else
								cost[p] += q2*bx2/B_rem[path[p][num_hops[p]]][indx1]; // cost function specific

							num_hops[p]++;
						    path[p][num_hops[p]] = indx1;

						}
					}
				}

				// CHOOSE PATH ...
				for (p=0; p<path_num; p++)
				{
					if ((cost_total - cost[p]) > threshold) //(cost[p] < cost_total))
					{
						cost_total = cost[p];
						sel_path = p;
					}
				}

				// UPDATE BANDWIDTH RESOURCES ...
				if (cost_total < (infinite - 1))
				{
					for (n=0; n<num_hops[sel_path]; n++)
						update(bx2, B_rem, path[sel_path][n], path[sel_path][n+1]);
				}

			} // if (path_num > 0)

		} // else if (bx2 > 0)
	}

	return cost_total;
}
