// greedy mapping algorithm:
// forward mapping approach (t-mapping is backward approach)
// the function calls another function that defines the cost function

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "crm.h"
#include "algorithms.h"

#define UPDATE_COSTS


extern float P[Nmax];			// processing powers of N processors per resource model
extern float B[Nmax][Nmax];		// System bandwidth matrices

extern float m_sort[Mmax];			// applications's processing requirements
extern float b[Mmax][Mmax];			// required bandwidths matrix

extern int N;

extern int P_m[Mmax];

float g_mapping(int M)
{
	// general
	int i,j,k,z,u,v;			// loop indices
	float m[Mmax];				// processing requirements

	float P_tempG[Nmax];			// remaining processing powers temporary-Global
	float P_tempL[Nmax][Nmax];		// remaining processing powers temporary-Local
	float P_aux;

    float B_tempG[Nmax][Nmax];		// remaining bandwidth capacities temporary-Global
	float B_tempL[Nmax][Nmax][Nmax];		// remaining bandwidths capacities temporary-Local
	
	float pp_cost = 0;			// partial path cost (computation + communication)
	float cost_aux = 0;			// minimum partial cost (computation + communication)
	float cost_g;				// g-mapping cost
	int indx;					// local index

	if (N<1 || N>Nmax) {
		printf("G-MAPPING ERROR. Invalid number of processors (%d, max %d)\n",N,Nmax);
		return 0;
	}
	
//	printf("G-MAPPING WITH %d processors\n",N);

	
	// intitialization of processing requirements
	for (i=0; i<M; i++)
		m[i] = m_sort[i];

	// more intitializations...
	for (k=0; k<N; k++)
        P_tempG[k] = P[k];

	for (u=0; u<N; u++)
		for (v=0; v<N; v++)
			B_tempG[u][v] = B[u][v];
	
	for (i=0; i<M; i++)
		P_m[i] = -1;		// a P_m[i] = -1 indicates that m[i] is not mapped

	// m[0] mapped to Pmax. Compute cost & update processing power:
	P_m[0] = 0;							// first assign process m[0] to processor P[0]
	P_aux = P_tempG[0];
	for (k=1; k<N; k++)
	{
		if (P_tempG[k] > P_aux)
		{
			P_m[0] = k;					// assign m[0] to P[i]
			P_aux = P_tempG[k];
		}
	}

	cost_g = cost_comp(m[0], P_tempG, P_m[0]);	// computation cost for the pre-mapping of m[0] to P_m[0]

	if (cost_g < (infinite - 1))	// m[0] is feasibly mapped to P[0]
	{
		for (i=1; i<M; i++)			// processes m[i]: m[1]..m[M-1]
		{
			cost_aux = infinite;	// initialize with infinite (infeasibility supposed)
			indx = -1;				// indx = -1 means that m[i] is not (feasibly) mapped, i.e. we suppose infeasibility

			for (j=0; j<N; j++)		// processors '0'..'N-1'
			{
				// COST OF COMPUTATION
				// ==============================================================================
				// initialization of remaining processing powers
				for (k=0; k<N; k++)
					P_tempL[j][k] = P_tempG[k];

				pp_cost = cost_comp(m[i], P_tempL[j], j); // computation cost for the pre-mapping of m[i] to P_tempL[j][j]

				if (pp_cost < (infinite - 1))
				{
				    // COST OF COMMUNICATION
					// ==============================================================================
					// initialization of remaining bandwidth capacities
					for (u=0; u<N; u++)
						for (v=0; v<N; v++)
							B_tempL[j][u][v] = B_tempG[u][v];
            
					for (z=0; z<i; z++)				// processes m[z]: m[0]..m[i-1]
					{
						if (P_m[z] == j)			// m[z] is pre-mapped to the same processor as m[i]
							;
						else						// m[z] and m[i] are pre-mapped to different processors
						{
							pp_cost += cost_comm(b[z][i], b[i][z], B_tempL[j], P_m[z], j);	// communication cost for the pre-mapping of m[i] to P_tempL[j][j]

							if (pp_cost > (infinite - 1))
							{
								pp_cost = infinite;
								break;
							}
						}
					}

					// PATH SELECTION:
					if ((cost_aux - pp_cost) > threshold)	// correct comparison of two floating point numbers ('threshold' > 0)
					{
						cost_aux = pp_cost;
						indx = j;
					}

				} // if (pp_cost < (infinite - 1))

			} // for (j=0; j<N; j++)		// processors '0'..'N-1'
		
			if (cost_aux < (infinite - 1))	// at least one feasible t-node at step i
			{
		        cost_g += cost_aux;			// accumulating the g-mapping cost
				P_m[i] = indx;				// mapping of m[i] to P_m[i]

				// updating processing powers
				for (k=0; k<N; k++)
					P_tempG[k] = P_tempL[indx][k];
	
				// updating bandwidth capacities
				for (u=0; u<N; u++)
					for (v=0; v<N; v++)
						B_tempG[u][v] = B_tempL[indx][u][v];
			}
			else							// all t-nodes at step i are infeasible
			{
				cost_g = infinite;
				break;
			}

		}	// for (i=1; i<M; i++) // processes m[i]: m[1]..m[M-1]

	} // if (cost_g < (infinite - 1))	// m[0] is feasibly mapped to P[0]

	// final mapping
	//if (cost_g < (infinite - 1))
	//	printf("\ng-mapping   (cost = %f):\t", cost_g);
	//else
	//	printf("\ng-mapping   (cost = %.0f):\t", cost_g);
	//for (i=0; i<M; i++)
	//	printf("%d\t", P_m[i]);
	//printf("\n");

	// save remaining processing and bandwidth capacities for the next application
#ifdef UPDATE_COSTS
	if (cost_g < (infinite-1))
	{
		for (j=0; j<N; j++)
			P[j] = P_tempG[j];

		for (i=0; i<N; i++)
			for (j=0; j<N; j++)
				B[i][j] = B_tempG[i][j];
	}
#endif
	//printf("\n%f\n", cost_g);
	return cost_g;
}
