/** @file hwman.c
 * @brief PHAL Hardware Manager SW Daemon Implementation
 * 
 * 
 */

/* standard libraries */
#include <stdlib.h>
#include <string.h>

#include "phal_hw_api.h"

#include "phid.h"
#include "phal_daemons.h"

/* common objects */
#include "pkt.h"
#include "daemon.h"

/* include command declarations */
#include "swload_cmds.h"
#include "hwman_cmds.h"

/* main function declaration */
void hwman_run();

#ifdef RUN_AS_PROC
int main()
{
	hwman_run();
}
#endif

int hwman_init(void);

void hwman_run()
{
	daemon_o daemon;
	int n;

	if (!hwman_init ()) {
		xprintf ("HWMAN: Error initating\n");
		exit(-1);
	}

	daemon = daemon_new(DAEMON_HWMAN);
	if (!daemon) {
		xprintf ("HWMAN: Error creating daemon\n");
		exit(-1);
	}
	

	/** Configure input commands
	 */

	if (daemon_addcmd(daemon, HWMAN_MAPOBJ, hwman_incmd_swmapper) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, HWMAN_ADDCPU, hwman_incmd_addcpu) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, HWMAN_ADDITF, hwman_incmd_additf) < 0) {
		exit(-1);
	}

	xprintf ("HWMAN: Init with PID=%d PPID=%d\n", getpid(), getppid());

	/** main daemon loop 
	 * while packets in queue, process them
	 */
	while (1) {

		do {
			n = daemon_rcvpkt(daemon);
			if (n>0) {
				daemon_processcmd(daemon);
			}
		} while (n>0);
		if (n < 0) {
			break;
		}
		hwapi_relinquish_daemon();
	}

	daemon_delete(&daemon);
	
	exit(-1);
}
