#include <stdlib.h>
#include <string.h>

#include "phal_hw_api.h"

#include "phid.h"
#include "phal_daemons.h"

/* common objects */
#include "set.h"
#include "str.h"
#include "var.h"
#include "app.h"
#include "rtcobj.h"
#include "phobj.h"
#include "phitf.h"
#include "pe.h"
#include "xitf.h"
#include "pkt.h"
#include "daemon.h"

#include "swload_cmds.h"
#include "hwman_cmds.h"
#include "frontend_cmds.h"
#include "bridge_cmds.h"

#include "crm/crm.h"
#include "crm/crm_functions.h"
#include "crm/global_variables.c"

//#define DEB

#define MAX_SLAVE_PE		5

struct bw_matrix_itf
{
	int loc_id;
	int opp_id;
	float B;
};

struct bw_matrix_itf BW[MAX_SLAVE_PE][MAX_SLAVE_PE];

/* variables from crm */

/* resource model */
extern float P[Nmax]; /* Nmax number of proc's. constant by now */
extern float B[Nmax][Nmax];

/* application model */
extern float b[Mmax][Mmax];
extern float m_sort[Mmax];
float m_unsort[Mmax];

/* final assignment */
extern int P_m[Mmax];

/* nof processors */
extern int N;

Set_o pedb = NULL;

int hwman_init ()
{
	pedb = Set_new(0, NULL);
	
	return pedb?1:0;
}

int calc_m_vector(Set_o objects)
{
	phobj_o obj;
	int i;

	assert (objects);
		
	for (i = 0; i < Set_length (objects); i++) {
		obj = Set_get (objects, i);
		/** @todo copy all rtcobj structure, more params than proc_req
		 */
		m_unsort[i]=obj->rtc->mips_req;
	}
		
	memcpy(m_sort, m_unsort, i*sizeof(float));
}

int calc_b_matrix(Set_o objects)
{
	int i, j, k;
	phobj_o obj_r, obj_w;
	phitf_o itf;
	
	assert (objects);
	
	for (i = 0; i < Set_length (objects); i++) {
		obj_r = Set_get (objects, i);
		for (j = 0; j < Set_length (obj_r->itfs); j++) {
			itf = Set_get (obj_r->itfs, j);
			if (phitf_iswriteonly (itf)) {
				for (k = 0; k < Set_length (objects); k++) {
					obj_w = Set_get (objects, k);
					if (!str_cmp (obj_w->objname, itf->remoteobjname)) {
						/** @todo copy all rtcitf structure, more params than just bw_req
						 */
						b[i][k] = itf->bw_req;
					}
				}
			}
		}
	}
	return 1;
}

int calc_P_vector (void)
{
	int i;
	pe_o pe;
	
	for (i = 0; i < Set_length (pedb); i++) {
		pe = Set_get (pedb, i);
		P[i] = pe->C;
		pe->P_idx = i;
	}
	N = i;
}

int calc_BW_matrix(void)
{
	int i, j, k;
	pe_o src_pe, dst_pe;
	xitf_o src_itf, dst_itf;

	memset(BW, 0, sizeof(struct bw_matrix_itf) * MAX_SLAVE_PE * MAX_SLAVE_PE);

	for (i = 0; i < Set_length (pedb); i++) {
		src_pe = Set_get (pedb, i);
		src_pe->BW_idx = i;
		for (j = 0; j < Set_length (pedb); j++) {
			if (i == j) {
				BW[i][i].loc_id=0;
				BW[i][i].opp_id=0;
				BW[i][i].B=(float) 100000;
				B[i][i]=(float) 100000;				
			} else {
				dst_pe = Set_get (pedb, j);	
				dst_itf = Set_find (src_pe->xitfs, &dst_pe->id, xitf_findremotepe);
				if (dst_itf) {
					BW[i][j].opp_id = dst_itf->xitf.id;
					BW[i][j].loc_id = dst_itf->remote_id;
					BW[i][j].B = dst_itf->xitf.BW;
					if (!B[i][j])
						B[i][j] = dst_itf->xitf.BW;
				}
			}
		}
	}
	
	return 1;
}

void print_m_vector(int len)
{
	int i;
	xprintf ("-- m vector --\n");
	for (i=0;i<len;i++) {
		xprintf ("%.2f\t", m_unsort[i]);
	}
	xprintf("\n");
}

void print_b_matrix(int len)
{
	int i,j;
	xprintf ("-- b matrix --\n");
	for (i=0;i<len;i++) {
		for (j=0;j<len;j++) {
			xprintf("%.2f\t", b[i][j]);
		}
		xprintf("\n");
	}
	xprintf("\n");
}

void print_P_vector(int len)
{
	int i;
	xprintf ("-- P vector --\n");
	for (i=0;i<len;i++) {
		xprintf ("%.2f\t", P[i]);
	}
	xprintf("\n");
}

void print_Pres_vector(int len)
{
	int i;
	xprintf ("-- P_m resulting vector --\n");
	for (i=0;i<len;i++) {
		xprintf ("%d\t", P_m[i]);
	}
	xprintf("\n");
}
void print_BW_matrix(int len)
{
	int i,j;
	xprintf ("-- BW matrix --\n");
	for (i=0;i<len;i++) {
		for (j=0;j<len;j++) {
			xprintf("%.2f\t", BW[i][j].B);
		}
		xprintf("\n");
	}
	xprintf("\n");
}

void one_mapping (int len)
{
	print_b_matrix(len);
	print_m_vector(len);
	print_BW_matrix(Set_length(pedb));
	print_P_vector(Set_length(pedb));
	P_m[0] = 1;
	P_m[1] = 0;
}


Set_o do_mapping (app_o app)
{
	Set_o app_dist;
	phobj_o obj_r, obj_w;
	phitf_o itf_r, itf_w;
	pe_o pe;
	int pe_r, pe_w;
	app_o dest_app, src_app;
	int i, j, k;
	float cost;
	
	calc_b_matrix(app->objects);
	calc_m_vector(app->objects);
	
//	one_mapping (Set_length(app->objects));
	print_b_matrix(Set_length(app->objects));
	print_m_vector(Set_length(app->objects));
	print_BW_matrix(Set_length(pedb));
	print_P_vector(Set_length(pedb));
	cost=g_mapping(Set_length (app->objects));
	
	xprintf ("HWMAN: Mapped with cost %.2f\n", cost);
	
	print_Pres_vector(Set_length(app->objects));
	
	app_dist = Set_new (0, NULL);
	if (!app_dist) {
		return NULL;
	}
	for (i = 0; i < Set_length (pedb); i++) {
		dest_app = app_new ();
		if (!dest_app) {
			Set_destroy(&app_dist, app_xdelete);
			return NULL;
		}
		Set_put (app_dist, dest_app);
	}

	/* Distribuite objects according to P_m[] vector and position of pe in that vector
	 */
	for (i = 0; i < Set_length (app->objects); i++) {
		dest_app = Set_get (app_dist, P_m[i]);
		pe = Set_find (pedb, &P_m[i], pe_findPidx);
		if (!pe) {
			Set_destroy(&app_dist, app_xdelete);
			return NULL;
		}
		obj_w = Set_get (app->objects, i);
		if (!obj_w) {
			Set_destroy(&app_dist, app_xdelete);
			return NULL;
		}
		obj_w->pe_id = pe->id;
		Set_put (dest_app->objects, obj_w);
	}
	
	
	for (i = 0; i < Set_length (app_dist); i++) {
		src_app = Set_get (app_dist, i);
		for (j = 0; j < Set_length (src_app->objects); j++) {
			obj_w = Set_get (src_app->objects, j);
			for (k = 0; k < Set_length (obj_w->itfs); k++) {
				itf_w = Set_get (obj_w->itfs, k);
				if (phitf_iswriteonly (itf_w) && !itf_w->xitf_id) {
					obj_r = Set_find (app->objects, str_str(itf_w->remoteobjname), phobj_findobjname);
					if (!obj_r) {
						xprintf ("HWMAN: Error remote object %s not found\n", str_str (itf_w->remoteobjname));
						Set_destroy(&app_dist, app_xdelete);
						return NULL;
					}
					if (obj_r->pe_id != obj_w->pe_id) {						
						itf_r = Set_find (obj_r->itfs, str_str(itf_w->remotename), phitf_findname);
						if (!itf_r) {
							xprintf ("HWMAN: Error remote object %s does not have any itf with name %s\n", str_str (itf_w->remoteobjname), str_str (itf_w->remotename));
							Set_destroy(&app_dist, app_xdelete);
							return NULL;
						}
						
						pe = Set_find (pedb, &obj_w->pe_id, pe_findid);
						if (!pe) {
							Set_destroy(&app_dist, app_xdelete);
							return NULL;
						}
						pe_w = pe->BW_idx;
						pe = Set_find (pedb, &obj_r->pe_id, pe_findid);
						if (!pe) {
							Set_destroy(&app_dist, app_xdelete);
							return NULL;
						}
						pe_r = pe->BW_idx;
						
						/** !!!!!!!!!!!! */
						itf_w->xitf_id= BW[pe_r][pe_w].loc_id;
						itf_w->remote_obj_id = obj_r->obj_id;
						itf_w->remote_id = itf_r->id;
						
						itf_r->xitf_id = BW[pe_r][pe_w].opp_id;
						itf_r->remote_obj_id = obj_w->obj_id;
						itf_r->remote_id = itf_w->id;
												
					}
				}
			}
		}
	}

	return app_dist;
}


int hwman_incmd_additf (daemon_o daemon)
{
	pkt_o pkt;
	pe_o pe;
	xitf_o xitf;
	
	assert (daemon);
	pkt = daemon_pkt (daemon);
	assert (pkt);
		
	pe = Set_find (pedb, pkt_getval (pkt, FIELD_PEID), pe_findid);
	if (!pe) {
		daemon_sendto (daemon, HWMAN_ADDITF, 0, DAEMON_HWMAN);
	} else {
		xitf = pkt_get (pkt, FIELD_XITF, xitf_newfrompkt);
		if (!xitf) {
			xprintf ("HWMAN: Error in ident packet\n");
			return 0;
		}
		Set_put (pe->xitfs, xitf);
		xprintf ("HWMAN: Processed new itf BW=%.2f remote %d\n", xitf->xitf.BW, xitf->remote_pe);
		calc_BW_matrix ();
		
		print_P_vector(Set_length(pedb));
		print_BW_matrix(Set_length(pedb));
	}
	
	return 1;
}

int send_total_cpu (daemon_o daemon) 
{
	pkt_o pkt;
	pe_o pe, tmppe;
	int i, sum_cpu;
	
	struct hwapi_cpu_i cpu_i;
	
	assert (daemon);
	pkt = daemon_pkt (daemon);
	assert (pkt);
	
	pe = pe_new ();
	if (!pe) {
		return 0;
	}
	
	sum_cpu = 0;
	for (i = 0; i < Set_length (pedb); i++) {
		tmppe = Set_get (pedb, i); 
		sum_cpu += tmppe->C;
	}
	
	hwapi_hwinfo_cpu (&cpu_i);
	
	pe->id = cpu_i.pe_id;
	pe->C = sum_cpu;
	pe->is_hwman = 1;
	/** ... */
	
	pkt_clear (pkt);
	
	pkt_put (pkt, FIELD_PE, pe, pe_topkt);
	xprintf ("HWMAN: Sending total cpu upwards\n");
	daemon_sendto (daemon, HWMAN_ADDCPU, 1, DAEMON_HWMAN);
	
	return 1;
}

void newson_error ()
{
	/** @todo errors
	 */
}


void swmapper_error ()
{
	/** @todo errors
	 */
}

void send_sons_ident (daemon_o daemon)
{
	pe_o pe;
	int i;
	pkt_o pkt;
	
	assert (daemon);
	
	pkt = daemon_pkt(daemon);
	
	pkt_clear(pkt);
	
	for (i = 0; i < Set_length (pedb); i++) {
		pe = Set_get (pedb, i);
		xprintf ("HWMAN: Sending indent packet to 0x%x\n", pe->id);
		daemon_sendto(daemon, BRIDGE_IDENT, pe->id, DAEMON_BRIDGE);
	}
}

int hwman_incmd_addcpu (daemon_o daemon)
{
	pkt_o pkt;
	
	pe_o pe, tmppe;
	
	assert (daemon);
	pkt = daemon_pkt (daemon);
	assert (pkt);
	
	
	tmppe = pkt_get (pkt, FIELD_PE, pe_newfrompkt);
	if (!tmppe) {
		xprintf ("HWMAN: Error in packet, PE info not found\n");
		newson_error (daemon);
		return 0;
	}
		
	/* overwrite existing one with same id, if any */
	pe = Set_find (pedb, &tmppe->id, pe_findid);
	if (pe) {		
		xprintf ("HWMAN: CPU updated for PE 0x%x (%.2f MIPS)\n", pe->id, tmppe->C);
		Set_remove (pedb, pe);
		pe_delete (&pe);
	} else { 
		xprintf ("HWMAN: New CPU: PE 0x%x, %.2f MIPS\n", tmppe->id, tmppe->C);
	}
	
	Set_put (pedb, tmppe);
	
	calc_P_vector ();
	
	pkt_clear (pkt);
	
	/* send new info to parent */
	send_total_cpu (daemon);

	/* send ident broadcast to sons */
	send_sons_ident (daemon);
	
	return 1;
}

int hwman_incmd_swmapper(daemon_o daemon)
{
	pkt_o pkt;
	app_o app, tmpapp;
	Set_o app_dist;
	phobj_o tmpobj;
	pe_o pe;
	int i;
	
	assert (daemon);
	pkt = daemon_pkt (daemon);
	assert (pkt);
	
	app = pkt_get (pkt, FIELD_APP, app_newfrompkt);
	if (!app) {
		xprintf ("HWMAN: Error received invalid packet.\n");
		swmapper_error (daemon);
		return 0;
	}

	app_dist = do_mapping (app);
	if (!app_dist) {
		app_delete (&app);
		xprintf ("HWMAN: Error mapping software\n");
		swmapper_error (daemon);
		return 0;
	}
	
	for (i = 0; i < Set_length (app_dist); i++) {
		tmpapp = Set_get (app_dist, i);
		tmpobj = Set_get (tmpapp->objects, 0);
		if (tmpobj) {
			pe = Set_find (pedb, &tmpobj->pe_id, pe_findid);
			if (!pe) {
				xprintf ("HWMAN: Error can't find pe 0x%x\n", tmpobj->pe_id);
			}
			pkt_clear (pkt);
			pkt_put (pkt, FIELD_APP, tmpapp, app_topkt);
			
			if (pe->is_hwman) {
				daemon_sendto (daemon, HWMAN_MAPOBJ, pe->id, DAEMON_HWMAN);			
			} else {
				daemon_sendto (daemon, SWLOAD_LOADOBJ, pe->id, DAEMON_SWLOAD);
			}
		}
	}
	app_delete (&app);
	Set_destroy (&app_dist, app_sdelete);
	
	return 1;
}

