/**	PHAL STATS SW Daemon Implementation
 * 
 * FILE: stats.c
 * 
 * Daemon type: Sensor
 * 
 * Communicates with: STATS MANAGER
 * 
 * Functions: This sensor manages statistics variables and 
 * 	initialization parameter values for the objects.
 * 	
 * 	In one side, it comunicates with every object (through
 * 	an interface for initialization procedures and through
 * 	hwapi services during run-time).
 * 	
 * 	In the other, it comunicates with statsAGER initializating
 * 	or providing/modifying values under request.
 */

/* standard libraries */
#include <stdlib.h>
#include <string.h>

#include "phal_hw_api.h"

#include "phid.h"
#include "phal_daemons.h"

/* common objects */
#include "pkt.h"
#include "daemon.h"

/* include command declarations */
#include "stats_cmds.h"

#include "stats.h"

/* main function declaration */
void stats_run();

#ifdef RUN_AS_PROC
int main()
{
	stats_run();
}
#endif

/** STATS MANAGER DAEMON MAIN FUNCTION 
 *
 */
void stats_run()
{
	daemon_o daemon;
	int n;

	daemon = daemon_new(DAEMON_STATS);
	if (!daemon) {
		xprintf ("stats: Error creating daemon\n");
		exit(-1);
	}

	/** Configure input commandsA
	 */

	if (daemon_addcmd(daemon, STATS_STINITOK, stats_incmd_initack) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, STATS_STINITERR, stats_incmd_initack) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, STATS_PMINITOK, stats_incmd_initack) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, STATS_PMINITERR, stats_incmd_initack) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, STATS_LOGINITOK, stats_incmd_initack) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, STATS_LOGINITERR, stats_incmd_initack) < 0) {
		exit(-1);
	}

	if (daemon_addcmd(daemon, STATS_STSET, stats_incmd_stact) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, STATS_STGET, stats_incmd_stact) < 0) {
		exit(-1);
	}

	if (daemon_addcmd(daemon, STATS_PMVALOK, stats_incmd_pmval) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, STATS_PMVALERR, stats_incmd_pmval) < 0) {
		exit(-1);
	}

	xprintf ("STATS: Init with PID=%d PPID=%d\n", getpid(), getppid());

	/** main daemon loop 
	 */
	while (1) {
		do {
			n = daemon_rcvpkt(daemon);
			if (n > 0) {
				daemon_processcmd(daemon);
			}
		} while (n);
		if (n < 0) {
			break;
		}

		stats_background(daemon);
		hwapi_relinquish_daemon();
	}

	daemon_delete(&daemon);

	exit(-1);
}
