/**
 *
 *
 *	PHAL Stats Manager SW Daemon Implementation
 *
 * FILE: statsman_cmds.c
 *
 * Description: Here is where functions answering COMMANDS are 
 *	implemented
 *
 * @todo Comment input commands
 */

/* standard libraries */
#include <stdlib.h>
#include <string.h>

#include "phid.h"
#include "phal_daemons.h"

#include "phal_hw_api.h"

/* common objects */
#include "set.h"
#include "str.h"
#include "var.h"
#include "rtcobj.h"
#include "phobj.h"
#include "log.h"
#include "pkt.h"
#include "daemon.h"

/* commands */
#include "stats_cmds.h"
#include "statsman_cmds.h"
#include "cmdman_cmds.h"

#include "statsman_parser.h"

char aux_str[128];

/* This is the main database for objects */
Set_o stm_db= NULL;

/* function to initialize database */
int statsman_init(void)
{
	stm_db = Set_new(0, NULL);
	if (!stm_db) {
		xprintf("STATSMAN: Error creating main database\n");
		return -1;
	}
	return 1;
}

void st_pm_init_error (daemon_o daemon, char st_pm)
{
	pkt_o pkt;
	objid_t obj_id;
	varid_t var_id;
	var_o var;
	
	pkt = daemon_pkt(daemon);
	
	obj_id = (objid_t) pkt_getvalue(pkt, FIELD_OBJID);
	var = pkt_get(pkt, st_pm?FIELD_STAT:FIELD_PARAM, var_newfrompkt);
		
	pkt_clear (pkt);
	
	pkt_putvalue(pkt, FIELD_OBJID, (uint) obj_id);
	pkt_putvalue(pkt, st_pm?FIELD_STID:FIELD_PMID, var?var->id:0);		
	
	daemon_sendto(daemon, st_pm?STATS_STINITERR:STATS_PMINITERR,
		pkt_getsrcpe(pkt), 
		DAEMON_STATS);
	
	var_delete(&var);
}

/** statsman_incmd_init
 */
int statsman_incmd_init(daemon_o daemon)
{
	pkt_o pkt;
	phobj_o obj;
	var_o stat, x;
	log_o log;
	int pmid;

	assert(daemon);

	pkt = daemon_pkt(daemon);

	obj = Set_find(stm_db, pkt_getval(pkt, FIELD_OBJID), phobj_findid);
	if (obj == NULL) {
		/* if we did not found it, create a new one */
		obj = phobj_new();
		if (!obj) {
			xprintf("STATSMAN: Error creating object\n");
			st_pm_init_error(daemon, 1);
			return 0;
		}
		char *appname = pkt_getval(pkt, FIELD_APPNAME);
		if (appname) {
			str_set(obj->appname, appname);
		}
		char *objname = pkt_getval(pkt, FIELD_OBJNAME);
		if (objname) {
			str_set(obj->objname, objname);
		}
		obj->obj_id = (objid_t) pkt_getvalue(pkt, FIELD_OBJID);
		obj->pe_id = (peid_t) pkt_getsrcpe(pkt);
		Set_put(stm_db, obj);
	}

	switch (pkt_getcmd(pkt)) {
	case STATSMAN_STINIT:
		stat = pkt_get(pkt, FIELD_STAT, var_newfrompkt);
		if (!stat) {
			xprintf("STATSMAN: Error can't create stat\n");
			st_pm_init_error(daemon, 1);
			return 0;
		}
		/* first check it does not exists already */
		x = Set_find(obj->stats, &stat->id, var_findid);
		if (x) {
			Set_remove(obj->stats, x);
			var_delete(&x);
		}
		Set_put(obj->stats, stat);

		pkt_clear(pkt);

		pkt_putvalue(pkt, FIELD_OBJID, (uint) obj->obj_id);
		pkt_putvalue(pkt, FIELD_STID, (uint) stat->id);

		daemon_sendto(daemon, STATS_STINITOK,
			pkt_getsrcpe(pkt), DAEMON_STATS);
		break;

	case STATSMAN_PMINIT:
		if (!statsman_parser_params(obj)) {
			
			xprintf("STATSMAN: Error parsing object's parameter file\n");
			st_pm_init_error(daemon, 0);
			return 0;
		}

		pmid = pkt_getvalue(pkt, FIELD_PMID);

		pkt_clear(pkt);

		pkt_putvalue(pkt, FIELD_OBJID, obj->obj_id);
		pkt_putvalue(pkt, FIELD_PMID, pmid);

		daemon_sendto(daemon, STATS_PMINITOK,
			pkt_getsrcpe(pkt), DAEMON_STATS);
		
		break;
	case STATSMAN_LOGINIT:
		/* ensure it does not already exist */
		log = Set_find (obj->logs, pkt_getval (pkt, FIELD_LOGID), log_findid);
		if (log) {
			xprintf("STATSMAN: Error creating log: Object %s, log %d already exists.\n",
					str_str(obj->objname), pkt_getvalue(pkt, FIELD_LOGID));
			daemon_sendto(daemon, STATS_LOGINITERR,
				pkt_getsrcpe(pkt), DAEMON_STATS);
			return -1;
		}
		
		char log_name[128];
		sprintf(log_name,"%s.%s",str_str(obj->objname), str_str(obj->appname));
		
		log = log_new(pkt_getvalue(pkt, FIELD_LOGID), log_name);		
		if (!log) {
			xprintf("STATSMAN: Error creating log: Can't create resource for %s\n", str_str(obj->objname));
			daemon_sendto(daemon, STATS_LOGINITERR,
				pkt_getsrcpe(pkt), DAEMON_STATS);
		} else {
			daemon_sendto(daemon, STATS_LOGINITOK,
				pkt_getsrcpe(pkt), DAEMON_STATS);			
			Set_put(obj->logs, log);
			
			//xprintf("STATSMAN: Log for object %s (%d) created.\n", str_str(obj->objname), pkt_getvalue(pkt, FIELD_LOGID));
		}		
		
		break;
	default:
		xprintf("error\n");
		return -1;
	}
	return 1;

}

/** statsman_incmd_close
 */
int statsman_incmd_close(daemon_o daemon)
{
	pkt_o pkt;
	phobj_o obj;
	var_o stat;
	log_o log;

	assert(daemon);

	pkt = daemon_pkt(daemon);

	obj = Set_find(stm_db, pkt_getval(pkt, FIELD_OBJID), phobj_findid);
	if (!obj) {
		xprintf("STATSMAN: Error closing: can't find object %d\n", *((objid_t*) pkt_getval(pkt, FIELD_OBJID)));
		return -1;
	}

	switch (pkt_getcmd(pkt)) {
	case STATSMAN_STCLOSE:

		stat = Set_find(obj->stats, pkt_getval(pkt, FIELD_STID),
				var_findid);
		if (!stat) {
			xprintf("STATSMAN: Error closing: can't find stat\n");
			return -1;
		}

		Set_remove(obj->stats, stat);
		var_delete(&stat);
		break;

	case STATSMAN_PMCLOSE:
		Set_destroy(&obj->params, var_xdelete);
		break;
	case STATSMAN_LOGCLOSE:
		log = Set_find (obj->logs, pkt_getval (pkt, FIELD_LOGID), log_findid);
		if (!log) {
			xprintf ("STATSMAN: Error log %d does not exists.\n", pkt_getvalue (pkt, FIELD_LOGID));
			return -1;
		}
		//xprintf("STATSMAN: Log for object %s (%d) closed.\n", str_str(obj->objname), pkt_getvalue(pkt, FIELD_LOGID));
		Set_remove(obj->logs, log);
		log_delete (&log);
		break;
	default:
		xprintf("error\n");
		break;
	}

	if (!Set_length(obj->stats) && !Set_length(obj->logs)) {
		obj = Set_remove(stm_db, obj);
		if (!obj) {
			xprintf("STATSMAN: Error closing: removing object\n");
			return -1;
		}
		phobj_delete(&obj);
	}

	return 1;
}

/** statsman_incmd_stact
 */
int statsman_incmd_stact(daemon_o daemon)
{

	pkt_o pkt;
	phobj_o obj;
	var_o stat;

	assert(daemon);

	pkt = daemon_pkt(daemon);

	obj = Set_find2(stm_db, pkt_getval(pkt, FIELD_OBJNAME), pkt_getval(pkt, 
			FIELD_APPNAME), phobj_findobjname, phobj_findappname);
	if (!obj) {
		xprintf("STATSMAN: Error with action: can't find object\n");
		daemon_sendto(daemon, CMDMAN_STVALERR, 0, DAEMON_CMDMAN);
		return 0;
	}

	stat = Set_find(obj->stats, pkt_getval(pkt, FIELD_STNAME),
			var_findname);
	if (!stat) {
		xprintf("STATSMAN: Error with action: can't find stat\n");
		daemon_sendto(daemon, CMDMAN_STVALERR, 0, DAEMON_CMDMAN);
		return 0;
	}

	if (pkt_getcmd(pkt) == STATSMAN_STSET) {
		var_setvalue(stat, pkt_getval(pkt, FIELD_STVALUE),
				pkt_getvalsz(pkt, FIELD_STVALUE));

		var_printstr(stat, aux_str);
		xprintf("value got ok (%s)", aux_str);

	}

	pkt_clear(pkt);
	pkt_putvalue(pkt, FIELD_OBJID, obj->obj_id);
	pkt_put(pkt, FIELD_STAT, stat, var_topkt);

	if (pkt_getcmd(pkt) == STATSMAN_STSET) {
		daemon_sendto(daemon, STATS_STSET, obj->pe_id, 
				DAEMON_STATS);
	} else {
		daemon_sendto(daemon, STATS_STGET, obj->pe_id, 
				DAEMON_STATS);
	}

	return 1;
}

/** statsman_incmd_stactack
 */
int statsman_incmd_stactack(daemon_o daemon)
{
	pkt_o pkt;
	phobj_o obj;
	var_o stat;

	assert(daemon);

	pkt = daemon_pkt(daemon);

	obj = Set_find(stm_db, pkt_getval(pkt, FIELD_OBJID), phobj_findid);
	if (!obj) {
		xprintf("STATSMAN: Error with actionACK: can't find object\n");
		return -1;
	}

	stat = Set_find(obj->stats, pkt_getval(pkt, FIELD_STID), var_findid);
	if (!stat) {
		xprintf("STATSMAN: Error with actionACK: can't find stat\n");
		return -1;
	}

	if (pkt_getcmd(pkt) == STATSMAN_STVALOK) {
		var_setvalue(stat, pkt_getval(pkt, FIELD_STVALUE),
				pkt_getvalsz(pkt, 
				FIELD_STVALUE));
	}

	pkt_clear(pkt);
	pkt_put(pkt, FIELD_APPNAME, obj->appname, str_topkt);
	pkt_put(pkt, FIELD_OBJNAME, obj->objname, str_topkt);
	pkt_put(pkt, FIELD_STAT, stat, var_topkt);

	switch (pkt_getcmd(pkt)) {
	case STATSMAN_STVALOK:
		var_printstr(stat, aux_str);
		xprintf("value got ok (%S)", aux_str);
		daemon_sendto(daemon, CMDMAN_STVALOK, 0, DAEMON_CMDMAN);
		break;

	case STATSMAN_STSETOK:
		daemon_sendto(daemon, CMDMAN_STSETOK, 0, DAEMON_CMDMAN);
		break;

	case STATSMAN_STVALERR:
		daemon_sendto(daemon, CMDMAN_STVALERR, 0, DAEMON_CMDMAN);
		break;

	case STATSMAN_STSETERR:
		daemon_sendto(daemon, CMDMAN_STSETERR, 0, DAEMON_CMDMAN);
		break;
	}

	return 1;
}

/** statsman_incmd_pmget
 */
int statsman_incmd_pmget(daemon_o daemon)
{
	pkt_o pkt;
	phobj_o obj;
	var_o param;
	varid_t pm_id;

	assert(daemon);

	pkt = daemon_pkt(daemon);

	obj = Set_find(stm_db, pkt_getval(pkt, FIELD_OBJID), phobj_findid);
	if (!obj) {
		xprintf("STATSMAN: Error getting param: can't find object\n");
		daemon_sendto(daemon, STATS_PMVALERR, pkt_getsrcpe(pkt), 
			DAEMON_STATS);
		return 0;
	}
	
	param = Set_find(obj->params, pkt_getval(pkt, FIELD_PMNAME),
			var_findname);
	if (!param) {
		xprintf("STATSMAN: Error Error getting param: can't find parameter\n");
		daemon_sendto(daemon, STATS_PMVALERR, pkt_getsrcpe(pkt), 
			DAEMON_STATS);
		return 0;
	}
	
	if ((vartype_t) pkt_getvalue(pkt, FIELD_PMTYPE) != var_type(param)) {
		xprintf("STATSMAN: Error Error getting param: object requested a different type "
				"from saved one.\n");
		daemon_sendto(daemon, STATS_PMVALERR, pkt_getsrcpe(pkt), 
			DAEMON_STATS);
		return 0;
	}

	pm_id = (varid_t) pkt_getvalue(pkt, FIELD_PMID);

	pkt_clear(pkt);

	pkt_putvalue(pkt, FIELD_OBJID, obj->obj_id);
	param->id = pm_id;
	pkt_put(pkt, FIELD_PARAM, param, var_topkt);

	daemon_sendto(daemon, STATS_PMVALOK, pkt_getsrcpe(pkt), 
		DAEMON_STATS);
	
	return 1;
}


/** statsman_incmd_logwrite
 */
int statsman_incmd_logwrite(daemon_o daemon)
{
	pkt_o pkt;
	phobj_o obj;
	log_o log;
	
	assert(daemon);

	pkt = daemon_pkt(daemon);

	obj = Set_find(stm_db, pkt_getval(pkt, FIELD_OBJID), phobj_findid);
	if (!obj) {
		xprintf("STATSMAN: Error writting log: can't find object\n");
		daemon_sendto(daemon, STATS_PMVALERR, pkt_getsrcpe(pkt), 
			DAEMON_STATS);
		return 0;
	}
	
	log = Set_find (obj->logs, pkt_getval (pkt, FIELD_LOGID), log_findid);
	if (!log) {
		xprintf ("STATSMAN: Error writting log: log %d does not exists.\n", pkt_getvalue (pkt, FIELD_LOGID));
		return -1;
	}
		
	log_Write(log, pkt_getval(pkt, FIELD_LOGTXT));
	
	return 1;
	
}


