/** @file statsman_parser.c 
 * @brief STATSMAN parsing functions implementation
 */

/* standard libraries */
#include <stdlib.h>
#include <string.h>

#include "phid.h"
#include "phal_daemons.h"
#include "stats.h"

#include "phal_hw_api.h"

/* common objects */
#include "set.h"
#include "str.h"
#include "var.h"
#include "rtcobj.h"
#include "phobj.h"
#include "pkt.h"
#include "daemon.h"

#include "statsman_parser.h"

#define PARAMS_BASE_DIR "./cfg"
#define PARAMS_EXT	".params"

#include "cfg_parser.h"

vartype_t
type_tophal (int type)
{
	switch (type) {
	case PARAM_STRING:
		return STAT_TYPE_CHAR;
	case PARAM_INT:
		return STAT_TYPE_INT;
	case PARAM_FLOAT:
		return STAT_TYPE_FLOAT;
	default:
		xprintf ("STATSMAN: ERROR Unsuported type %d\n", type);
		return 0;
	}
}

char filename[128];

int statsman_parser_params(phobj_o obj)
{
	Set_o varset;
	var_o var;
	char *buffer;
	int i, j;
	cfg_o cfg;
	sect_o sect;
	key_o key, name, value;
	int offset;
	int size, type;

	sprintf(filename, "statsman/%s/%s.params", str_str(obj->appname),
			str_str(obj->objname));

	offset = hwapi_res_parseall(filename, &buffer);
	if (offset < 0) {
		xprintf ("STATSMAN: Error opening file %s\n", filename);
		return 0;
	}

	cfg = cfg_new(buffer, offset);
	if (!cfg) {
		xprintf ("STATSMAN: Error parsing file %s", filename);
		free(buffer);
		return 0;
	}

	varset = obj->params;
	if (!varset) {
		xprintf ("STATSMAN: Error creating variables");
		free(buffer);
		return 0;
	}

	for (i = 0; i < Set_length(cfg_sections(cfg)); i++) {
		sect = Set_get(cfg_sections(cfg), i);

		if (strcmp(sect_title(sect), "parameter")) {
			xprintf ("STATSMAN: Error unknown section %s\n", sect_title (sect));
		}

		name = Set_find(sect_keys(sect), "name", key_findname);
		value = Set_find(sect_keys(sect), "value", key_findname);

		type = key_type(value);
		if (type == -1) {
			xprintf ("STATSMAN: Caution type not guessed, trying with integer...\n");
			type = PARAM_INT;
		}

		size = key_nofelems(value, type);
		if (size == -1) {
			xprintf ("STATSMAN: Error trying to guess size, aborting...\n");
			break;
		}

		if (name && value) {
			var = var_new((varsz_t) size,
					(vartype_t) type_tophal(type));
			if (!var) {
				xprintf ("STATSMAN: Error creating variable");
				break;
			}

			str_set(var->name, (char *) key_value(name, 0, 
					PARAM_STRING));
			
			if (!var_setvalue(var, key_value(value, size, type),
					size)) {
				xprintf ("STATSMAN: Error setting variable value\n");
				break;
			}

			Set_put(varset, var);
		} else {
			xprintf ("STATSMAN: Not enougth params\n");
			break;
		}
	}

	free(buffer);

	if (i < Set_length(cfg_sections(cfg))) {
		xprintf ("STATSMAN: Error some params were not parsed.\n");
	}

	cfg_delete(&cfg);

	return 1;
}

