/** @file swload.c
 * @brief PHAL Software Load SW Daemon Implementation
 *
 * Daemon type: Sensor
 *
 * Communicates with: SW MANAGER
 *
 * Functions: This sensor has two main tasks:
 *	1) Provide platform-specific information to the SWMANAGER daemon when requested. 
 *		This will enable to load the specific executable and under specific constraints. 
 *
 *	2) Create and assign local resources to each executable, under manager orders.
 *
 * Config Files:
 *	none
 */

/* standard libraries */
#include <stdlib.h>
#include <string.h>

#include "phal_hw_api.h" 

#include "phid.h"
#include "phal_daemons.h"

/* common objects */
#include "pkt.h"
#include "daemon.h"

/* include command declarations */
#include "swload_cmds.h"

/* main function declaration */
void swload_run();

#ifdef RUN_AS_PROC
int main()
{
	swload_run();
}
#endif

int swload_init(void);

/** STATS MANAGER DAEMON MAIN FUNCTION 
 *
 */
void swload_run()
{
	daemon_o daemon;
	int n;

	daemon = daemon_new(DAEMON_SWLOAD);
	if (!daemon) {
		xprintf ("stats: Error creating daemon\n");
		exit(-1);
	}

	if (daemon_addcmd(daemon, SWLOAD_LOADOBJ, swload_incmd_loadobj) < 0) {
		exit(-1);
	}

	if (daemon_addcmd(daemon, SWLOAD_EXECERR, swload_incmd_execerr) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, SWLOAD_LOADSTART, swload_incmd_loadstart) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, SWLOAD_LOADING, swload_incmd_loading) < 0) {
		exit(-1);
	}
	if (daemon_addcmd(daemon, SWLOAD_LOADEND, swload_incmd_loadend) < 0) {
		exit(-1);
	}

	if (swload_init() < 0) {
		exit(-1);
	}

	xprintf ("SWLOAD: Init with PID=%d PPID=%d\n", getpid(), getppid());

	/** main daemon loop 
	 */
	while (1) {

		do {
			n = daemon_rcvpkt(daemon);
			if (n>0) {
				daemon_processcmd(daemon);
			}
		} while (n>0);
		if (n < 0) {
			break;
		}

		hwapi_relinquish_daemon();
	}

	daemon_delete(&daemon);

	exit(-1);
}
