#include "rsol_code.h"
///////////////////////////////////
//// Galois Field Operations
///////////////////////////////////

/* Inicialitzacio lookup tables */
void init_exp_table (void)
{
	int i, z;
	int pinit,p1,p2,p3,p4,p5,p6,p7,p8;

	pinit = p2 = p3 = p4 = p5 = p6 = p7 = p8 = 0;
	p1 = 1;
	
	gexp[0] = 1;
	gexp[255] = gexp[0];
	glog[0] = 0;			/* shouldn't log[0] be an error? */
	

	//calcul de taula exp, utilitza un LFSR amb el polinomi /* x^8 + x^4 + x^3 + x^2 + 1 */
	for (i = 1; i < 256; i++) {
		pinit = p8;
		p8 = p7;
		p7 = p6;
		p6 = p5;
		p5 = p4 ^ pinit;
		p4 = p3 ^ pinit;
		p3 = p2 ^ pinit;
		p2 = p1;
		p1 = pinit;
		gexp[i] = p1 + p2*2 + p3*4 + p4*8 + p5*16 + p6*32 + p7*64 + p8*128;
		gexp[i+255] = gexp[i];	//pq duplica la taula? 
  	}
	

	//calcul de la taula inversa (log) // COM ?¿?
	for (i = 1; i < 256; i++) {
		for (z = 0; z < 256; z++) {
			if (gexp[z] == i) {
				glog[i] = z;
				break;
			}
		}
	}
}

/* multiplication using logarithms */
int gmult(int a, int b)
{
	int i,j;
	if (a==0 || b == 0) return (0);
	i = glog[a];
	j = glog[b];
	return (gexp[i+j]);
}


////////////////////////////////////////
///// OPERACIONS AMB POLINOMIS
////////////////////////////////////////
 
/* polynomial multiplication */
void mult_polys (int dst[], int p1[], int p2[])
{
	int i, j;
	int tmp1[MAXDEG*2];
	
	for (i=0; i < (MAXDEG*2); i++) dst[i] = 0;
	
	for (i = 0; i < MAXDEG; i++) {
		for(j=MAXDEG; j<(MAXDEG*2); j++) tmp1[j]=0;
		
		/* scale tmp1 by p1[i] */
		for(j=0; j<MAXDEG; j++) tmp1[j]=gmult(p2[j], p1[i]);

		/* and mult (shift) tmp1 right by i */
		for (j = (MAXDEG*2)-1; j >= i; j--) 
			tmp1[j] = tmp1[j-i];

		for (j = 0; j < i; j++) tmp1[j] = 0;
				
		/* add into partial product */
		for(j=0; j < (MAXDEG*2); j++) dst[j] ^= tmp1[j];
  	}
}

/* copia polinomis*/
void copy_poly (int dst[], int src[]) 
{
	int i;
	for (i = 0; i < MAXDEG; i++) 
		dst[i] = src[i];
}

/* posa polinomi a 0 */
void zero_poly (int poly[]) 
{
	int i;
	for (i = 0; i < MAXDEG; i++) 
		poly[i] = 0;
}

/* Calcul del G(X) */
void compute_genpoly (int nbytes, int genpoly[])
{
	int i, tp[256], tp1[256];
	
	/* multiply (x + a^n) for n = 1 to nbytes */

	zero_poly(tp1);
	tp1[0] = 1; //1·X^0 = 1

	for (i = 1; i <= nbytes; i++) {
		zero_poly(tp);
		tp[0] = gexp[i]; //a^n·X^0		/* set up x+a^n */
		tp[1] = 1; //1·X^1
			
		mult_polys(genpoly, tp, tp1);
		copy_poly(tp1, genpoly);
	}
}



////////////////////////////////////////
///// ENCODING
////////////////////////////////////////

/* Paraula codi resultant */
void build_codeword (unsigned char msg[], int nbytes, unsigned char dst[])
{
	int i;
		
	 //primer la part del msg
		
	for (i = 0; i < NPAR; i++) {			//despres hi afegeix la redundancia
		dst[i] = pBytes[NPAR-1-i];
  	}

	for (i = 0; i < nbytes; i++) dst[i+NPAR] = msg[i];
}

/* Reed-Salomon encoder a partir del model LFSR */
void encode_data (unsigned char msg[], int nbytes, unsigned char dst[])
{
	int i, LFSR[NPAR+1],dbyte, j;
		
	for(i=0; i < NPAR+1; i++) LFSR[i]=0; //Posa a zero el registre
	
	for (i = 0; i < nbytes; i++) {
		dbyte = msg[i] ^ LFSR[NPAR-1]; //Aixo es el feedback
		for (j = NPAR-1; j > 0; j--) {
			LFSR[j] = LFSR[j-1] ^ gmult(genPoly[j], dbyte); //un cop per cada registre-1
		}
		LFSR[0] = gmult(genPoly[0], dbyte); // Correspon al X^0
	}
	
	for (i = 0; i < NPAR; i++) 
		pBytes[i] = LFSR[i];	//copia el contingut del registre a pBytes
		
	build_codeword(msg, nbytes, dst);  //construeix la paraula codi	
}



void fillzero (unsigned char info_in[], unsigned char msg[],int * bitmod)
{

	int i;

	//fill msg with (K-K') 0's
	for (i=0; i<(K_LENGTH-*bitmod-1); i++)
		msg[i]=0;
	
	//copy info(bytes) into msg array
	for (i=0; i<*bitmod; i++)
		msg[(K_LENGTH-*bitmod-1)+i]=info_in[i];
	
	msg[K_LENGTH-1]=0; //tail 
	
	*bitmod+=1;	

//	for (i=0; i<K_LENGTH; i++)
//		printf("%d\n",msg[i]);

}



void delredundant (unsigned char code[],int difmod, int inmod)
{
	unsigned char tmp[N_LENGTH]; 
	int dif,i,j;	
	
	//Puncturing, redundancy to tmp
	for (i=0; i<difmod; i++)
		tmp[i]=code[i];

	dif=16-difmod;

	//shortened (0's are not copied in tmp)
	for (i=difmod,j=0; i<(inmod+difmod); i++,j++)
		tmp[i]=code[16+(K_LENGTH-inmod)+j];	
	
	//building the codeword
	for(i=0;i<(difmod+inmod);i++)
		code[i]=tmp[i];

}

